/*
*  attachdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "attachdlgs.h"
#include "utility.h"
#include "global.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TAttachDlg *AttachDlg;
const AnsiString attmsg =
"   If you aren't familiar with the way the mixmaster network handles "
"attachments, please take a look at the help below. It won't take long and it "
"will prevent confusion and possible loss of mail later.\n\n"
"Click [Browse] to select your file...";

/*"    We all need to send an attachment now and then, but mixmaster isn't very "
"good for that purpose. It's primarily a text messaging protocol with a max "
"message size of around 29K. When you attach a file, the required MIME encoding "
"will increase the file's size by about 30%. Messages larger messages than 29K "
"get split up into multiple messages. Everything is reassembled at the last "
"remailer before being delivered. The problem is, the more messages, the more "
"likely one or more won't make it to that last remailer. In that case the "
"message is eventually discarded.\n\n"
"    Large attachments are bad for the remailer network. They put a strain on "
"the system. Worse, for you, the larger the attachment the more mixmaster "
"messages will be required to transmit your file. A single mix messages holds "
"approx. 29K payload. Bigger payloads are split up. To exacerbate the problem, "
"the required MIME encoding will increase the size of your file by about 30%. \n\n"
"    I realize though, we all need to send an attachment now and then. Keep it "
"small as possible. When I'm processing the message while sending I'll let you "
"know if I encounter any problems requiring your intervention.";
*/
//---------------------------------------------------------------------------

AnsiString __fastcall GetFileToAttach(TForm *Owner)
{
   AnsiString filename;
   TAttachDlg *attachdlg = new TAttachDlg(Owner);
   attachdlg->ShowModal();
   if (attachdlg->ModalResult == mrOk)
      filename = attachdlg->OpenDialog->FileName;
   delete attachdlg;
   return filename;
}
//---------------------------------------------------------------------------

__fastcall TAttachDlg::TAttachDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   RichEdit1->Lines->Text = attmsg;
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   OpenDialog->InitialDir = ini->ReadString("settings", "lastattachpath", AppDir());
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TAttachDlg::BrowseBtnClick(TObject *Sender)
{
   if (OpenDialog->Execute())
   {
      TIniFile *ini = new TIniFile(AppIni());
      ini->WriteString("settings", "lastattachpath",
         ExtractFilePath(OpenDialog->FileName));
      delete ini;
      ModalResult = mrOk;
   }
}
//---------------------------------------------------------------------------

void __fastcall TAttachDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/attach.htm>qscshelp");
}
//---------------------------------------------------------------------------

