/*
*  draftdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "draftdlgs.h"
#include "utility.h"
#include "global.h"
#include "qsdbs.h"
#include "help.h"
#include "inputdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDraftDlg *DraftDlg;
//---------------------------------------------------------------------------

AnsiString __fastcall SelectDraft(TForm *Owner)
{
   AnsiString draft;
   TDraftDlg *draftdlg = new TDraftDlg(Owner);
   draftdlg->SaveName->Visible = false;
   draftdlg->Label2->Visible = false;
   draftdlg->ShowModal();
   if (draftdlg->ModalResult == mrOk)
   {
	  int x = draftdlg->DraftList->ItemIndex;
	  assert(x >= 0);
	  draft = draftdlg->DraftList->Items->Strings[x].Trim();
   }
   delete draftdlg;
   return draft;
}
//---------------------------------------------------------------------------

AnsiString __fastcall SaveDraftAs(TForm *Owner, AnsiString CurrName)
{
   AnsiString draft;
   TDraftDlg *draftdlg = new TDraftDlg(Owner);
   draftdlg->Label1->Caption = "Enter the name you want this draft saved as...";
   draftdlg->SaveName->Visible = true;
   draftdlg->Label2->Visible = true;
   draftdlg->SaveName->Text = CurrName;
   draftdlg->ActiveControl = draftdlg->SaveName;
   draftdlg->OkBtn->Enabled = true;
   draftdlg->ShowModal();
   if (draftdlg->ModalResult == mrOk)
   {
	  assert(draftdlg->SaveName->Text != "");
	  draft = draftdlg->SaveName->Text;
   }
   delete draftdlg;
   return draft;
}
//---------------------------------------------------------------------------

__fastcall TDraftDlg::TDraftDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "draftdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(DraftList->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   delete ini;
   //Label2->Left = SaveName->Left - (Label2->Width + 5);
   if (Font->Size > 11)
	  Label1->Top = 2;
   TStringList *items = new TStringList();
   DB->LoadDrafts(items);
   for (int i = 0; i < items->Count; i++)
	  DraftList->Items->Add(items->Strings[i]);
   delete items;
}
//---------------------------------------------------------------------------

__fastcall TDraftDlg::~TDraftDlg()
{
   if (IntToStr(Width)+IntToStr(Height) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "draftdlg", ini);
	  delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::Open1Click(TObject *Sender)
{
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::DraftListClick(TObject *Sender)
{
   int x = DraftList->ItemIndex;
   DeleteBtn->Enabled = x >= 0;
   RenameBtn->Enabled = x >= 0;
   OkBtn->Enabled = x >= 0 || SaveName->Visible;
   if (SaveName->Visible && x >= 0)
	  SaveName->Text = DraftList->Items->Strings[x].Trim();
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::FormShow(TObject *Sender)
{
   if (SaveName->Text.IsEmpty())
   {
	  int i = 1; while (DraftList->Items->IndexOf("draft"+IntToStr(i)) >= 0)
		 i++;
	  SaveName->Text = "draft"+IntToStr(i);
   }
   Label2->Top = SaveName->Top + 3;
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk && SaveName->Visible && SaveName->Text.IsEmpty())
   {
      MsgBox(this,
         "You need to enter a name to save the draft as, or click Cancel!   ",
		  MT_INFO, MB_OK);
      CanClose = false;
      return;
   }

   if (ModalResult == mrOk && !SaveName->Visible && DraftList->ItemIndex < 0)
   {
	  ModalResult == mrCancel;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::PopupMenu1Popup(TObject *Sender)
{
   int x = DraftList->ItemIndex;
   Delete1->Enabled = x >= 0;
   Rename1->Enabled = x >= 0;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------
void __fastcall TDraftDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::DraftListDrawItem(TWinControl *Control, int Index,
      TRect &Rect, TOwnerDrawState State)
{
   DraftList->Canvas->TextRect(Rect, Rect.Left + 5, Rect.Top - 1,
      DraftList->Items->Strings[Index]);
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::HelpClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/draftsdlg.htm>drfthelp");
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::Delete1Click(TObject *Sender)
{
   int x = DraftList->ItemIndex;
   assert(x >= 0);
   AnsiString name = DraftList->Items->Strings[x].TrimLeft();

   if (MsgBox(this,
      "Delete the draft '" + name + "'. Are you sure? ",
       MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
      DraftList->Items->Delete(x);
	  DB->DelDraft(name);
	  OkBtn->Enabled = SaveName->Visible;
	  RenameBtn->Enabled = false;
	  DeleteBtn->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDraftDlg::Rename1Click(TObject *Sender)
{
   assert(DraftList->ItemIndex >= 0);
   AnsiString newname, oldname = DraftList->Items->Strings[DraftList->ItemIndex];
   newname = GetUserInput(this, "Rename Draft", "Enter new name", oldname,
	  DraftList->Items);
   if (!newname.IsEmpty() && newname != oldname)
   {
      DraftList->Items->Delete(DraftList->ItemIndex);
      DraftList->ItemIndex = DraftList->Items->Add(newname);
      DB->RenDraft(oldname, newname);
   }
}
//---------------------------------------------------------------------------


