/*
*  global.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <inifiles.hpp>

#include "global.h"
#include "gpg.h"
#include "mixmaster.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
UINT WM_QSLMESSAGE; // assigned by RegisterWindowsMessage
AnsiString appdatadir = "";
AnsiString qsadatadir = "";
int NextX=0, NextY=0, OrgX=0, OrgY=0;
TList *windowlist = NULL;
//---------------------------------------------------------------------------

AnsiString __fastcall AppDir()
{
   return ExtractFilePath(Application->ExeName).LowerCase();
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppDataDir()
{
   return appdatadir; //AppDir();
}
//---------------------------------------------------------------------------

AnsiString __fastcall QsaDataDir()
{
   return qsadatadir; //AppDir();
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppIni()
{
   return AppDataDir() + ExtractFileName(ChangeFileExt(Application->ExeName, ".ini"));
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppDefsIni()
{
   return AppDir() + "qsl.defs.txt";
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppChm()
{
   return ChangeFileExt(Application->ExeName, ".chm");
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppChmWB()
{
   return AppDir() + "qsw.chm";
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppOldDB()
{
   return GetShortName(appdatadir).LowerCase() +
	  ExtractFileName(ChangeFileExt(Application->ExeName, ".db"));
}
//---------------------------------------------------------------------------
#ifdef QS2
AnsiString __fastcall AppDB()
{
   return GetShortName(appdatadir).LowerCase() +
	  ExtractFileName(ChangeFileExt(Application->ExeName, ".db"));
}
#else
AnsiString __fastcall AppDB()
{
   return GetShortName(appdatadir).LowerCase() + "qs.db";
}
#endif
//---------------------------------------------------------------------------

AnsiString __fastcall AppDBTmp()
{
   return AppDB() + ".tmp";
}
//---------------------------------------------------------------------------

AnsiString __fastcall AppLicense()
{
   return AppDir() + "copying.txt";
}
//---------------------------------------------------------------------------

AnsiString __fastcall DefNoteOpts()
{
   return IntToStr((int)DefNoteWordWrap)+","+IntToStr(DefNoteWidth)+","+
	  IntToStr(DefNoteHeight)+","+IntToStr(DefNoteColor)+",font="+
	  OptsDefEditorFont;
}
//---------------------------------------------------------------------------

void __fastcall GetNextWindowXY(TForm *Window)
{
   if (windowlist == NULL)
   {
	  windowlist = new TList();
	  windowlist->Add(Window);
#ifndef QS2
	  OrgX = Window->Left;
	  NextX = OrgX + 25;
	  OrgY = Window->Top;
	  NextY = OrgY + 25;
	  return;
#endif
   }
   if (!NextX)
   {
	  OrgX = Application->MainForm->Left + 25;
	  OrgY = Application->MainForm->Top + 25;
	  NextX = OrgX;
	  NextY = OrgY;
   }
   else
   {
	  if (NextY + Window->Height > Screen->Height - 25)
		 NextY = OrgY;
	  if (NextX + Window->Width > Screen->Width - 25)
		 NextX = OrgX;
   }
   Window->Left = NextX;
   Window->Top = NextY;
   windowlist->Add(Window);

   NextX += 25;
   NextY += 25;
}
//---------------------------------------------------------------------------

AnsiString __fastcall addbs(AnsiString path)
{
   int l = path.Length();
   if (l && path[l] != '\\')
	  path += "\\";
   return path;
}
//---------------------------------------------------------------------------

bool __fastcall SetApplicationDataDirectory()
{
   if (FileExists(AppDir()+"datadir.ini"))
   {
	  TIniFile *ini = new TIniFile(AppDir()+"datadir.ini");
	  AnsiString datadir = ini->ReadString("settings", "appdatadir", "");
	  delete ini;
	  if (datadir.IsEmpty() || !DirectoryExists(datadir))
	  {
		 MessageBox(NULL,
			"The 'appdatadir' directory specified in datadir.ini doesn't exist!",
			"ERROR", MB_OK);
		 return false;
	  }
	  appdatadir = datadir;
	  return true;
   }

   char path[MAXPATH + 1];
   AnsiString mydir = ExtractFilePath(Application->ExeName).LowerCase();

   path[0] = '\0';
   GetEnvironmentVariable("ProgramFiles", path, MAXPATH + 1);
   if (mydir.Pos(addbs(path).LowerCase()) == 1)
   {
	  goto appdata;
   }

   path[0] = '\0';
   GetEnvironmentVariable("ProgramFiles(x86)", path, MAXPATH + 1);
   if (mydir.Pos(addbs(path).LowerCase()) == 1)
   {
	  goto appdata;
   }

   path[0] = '\0';
   GetEnvironmentVariable("ProgramW6432", path, MAXPATH + 1);
   if (mydir.Pos(addbs(path).LowerCase()) == 1)
   {
	  goto appdata;
   }

   appdatadir = AppDir();
   gpgdatadir = GetShortName(appdatadir).LowerCase();
   gpgdir = gpgdatadir;
   if (!SetEnvironmentVariable("GNUPGHOME", gpgdatadir.c_str()))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'GNUPGHOME'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }
   if (!SetEnvironmentVariable("LANGUAGE", "en@quot.mo"))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'LANGUAGE'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }
   if (!SetEnvironmentVariable("MIX3PATH", MixDataDir().c_str()))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'MIX3PATH'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }
   return true;

appdata:

   if (GetEnvironmentVariable("AppData", path, MAXPATH + 1) > 0)
   {
	  appdatadir = addbs(path) + "quicksilver\\qsl\\";
	  qsadatadir = addbs(path) + "quicksilver\\qsa\\";
	  if (!DirectoryExists(qsadatadir))
		 qsadatadir = "";
	  if (qsadatadir.IsEmpty() && FileExists(appdatadir+"qsa.ini"))
		 qsadatadir = appdatadir;
   }
   else
   {
	  appdatadir = AppDir();
	  if (FileExists(appdatadir+"qsa.ini"))
		 qsadatadir = AppDir();
   }
   if (!DirectoryExists(appdatadir))
	  ForceDirectories(appdatadir);
   assert(DirectoryExists(appdatadir));

   gpgdir = GetShortName(AppDir());
   gpgdatadir = GetShortName(appdatadir);
   if (!SetEnvironmentVariable("GNUPGHOME", gpgdatadir.c_str()))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'GNUPGHOME'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }
   if (!SetEnvironmentVariable("LANGUAGE", "en@quot.mo"))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'LANGUAGE'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }
   if (!SetEnvironmentVariable("MIX3PATH", MixDataDir().c_str()))
   {
	  MessageBox(NULL,
		 AnsiString("I was unable to set the environment variable 'MIX3PATH'.\n"
		 "The error reported was:\n\n" + GetLastErrorStr()).c_str(),
		 "ERROR", MB_OK);
	  return false;
   }

   return true;
}
//---------------------------------------------------------------------------

