/*
*  httpmod.h 
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/
//---------------------------------------------------------------------------
#ifndef httpmodH
#define httpmodH
#undef SetPort
#include <classes.hpp>
#include <idsocks.hpp>
#include <IdBaseComponent.hpp>
#include <IdComponent.hpp>
#include <IdHTTP.hpp>
#include <IdTCPClient.hpp>
#include <IdTCPConnection.hpp>
#include <IdIOHandlerStack.hpp>
#include <IdSSL.hpp>
#include <IdIOHandler.hpp>
#include <IdIOHandlerSocket.hpp>
#include <IdSSLOpenSSL.hpp>
#include <IdLogEvent.hpp>

class TQSHTTP;
//---------------------------------------------------------------------------
bool __fastcall HTTPGetURL(TForm *Owner, const AnsiString &URL, TStrings *Proxy,
   TStrings *Log, TStrings *Page, AnsiString &ErrorMsg, bool *Cancel);

class TQSHTTPThread : public TThread
{
private:
protected:
	void __fastcall Execute();
public:
	TForm *Owner;
	TQSHTTP *HTTP;
	TStrings *page, *log;
	bool finished, success;
	AnsiString errormsg, TheURL;
	TIdSocksInfo *socksinfo;
	TIdSSLIOHandlerSocketOpenSSL *ssliohandler;
	__fastcall TQSHTTPThread(bool CreateSuspended);
	__fastcall TQSHTTPThread(TForm *Owner, const AnsiString &URL, TStrings *Proxy,
	   TStrings *Log, TStrings *Page, bool *Cancel);
	__fastcall virtual ~TQSHTTPThread();
	void __fastcall Log(AnsiString line);
};

class TQSLogEvent : public TIdLogEvent
{
	void __fastcall LogEventStatus(TComponent *ASender, const AnsiString AText);
	void __fastcall LogEventConnect(TIdConnectionIntercept *ASender);
	void __fastcall LogEventSend(TIdConnectionIntercept *ASender, TIdBytes &ABuffer);
	void __fastcall LogEventSent(TComponent *ASender, const AnsiString AText, const AnsiString AData);
	void __fastcall LogEventReceive(TIdConnectionIntercept *ASender, TIdBytes &ABuffer);
	void __fastcall LogEventReceived(TComponent *ASender, const AnsiString AText, const AnsiString AData);
	void __fastcall LogEventDisconnect(TIdConnectionIntercept *ASender);
public:
	TForm *Owner;
	TQSHTTP *http;
	TQSHTTPThread *thread;
	__fastcall TQSLogEvent(TComponent *owner);
};

class TQSHTTP : public TIdHTTP
{
	void __fastcall HTTPEventWork(TObject *ASender, TWorkMode AWorkMode, __int64 AWorkCount);
	void __fastcall HTTPEventWorkBegin(TObject *ASender, TWorkMode AWorkMode, __int64 AWorkCountMax);
	void __fastcall HTTPEventWorkEnd(TObject *ASender, TWorkMode AWorkMode);
public:
	bool *cancel;
	TStrings *log;
	TMemoryStream *Content;
	__fastcall TQSHTTP(TComponent *Owner);
	__fastcall virtual ~TQSHTTP();
};

//const UINT WM_SHOWSTATUS = WM_USER+11;
//const UINT WM_LOGLINE = WM_USER+12;
const UINT WM_PROGRESS = WM_USER+13;

const AnsiString     httpciphersuite = "ALL:!ADH:RC4+RSA:+SSLv3:@STRENGTH";
const TIdSSLVersions httpsslversions = TIdSSLVersions() << sslvTLSv1 << sslvTLSv1_1 << sslvTLSv1_2;
//---------------------------------------------------------------------------
#endif
