/*
*  logdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "logdlgs.h"
#include "qsdbs.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TLogDlg *LogDlg;
//---------------------------------------------------------------------------

void __fastcall ViewSMTPLog(TForm *Owner)
{
   TLogDlg *logdlg = new TLogDlg(Owner);
   logdlg->Caption = "SMTP Log";
   DB->LoadLog(SMTPLog, logdlg->log);
   logdlg->Viewer->Lines->AddStrings(logdlg->log);
   logdlg->Viewer->SelStart = logdlg->Viewer->Text.Length();
   logdlg->Viewer->Perform(EM_SCROLLCARET, 0, 0);
   logdlg->ShowModal();
   delete logdlg;
}
//---------------------------------------------------------------------------
#ifdef QS2
void __fastcall ViewPOP3Log(TForm *Owner)
{
   TLogDlg *logdlg = new TLogDlg(Owner);
   logdlg->Caption = "POP Log";
   DB->LoadLog(POP3Log, logdlg->log);
   logdlg->Viewer->Lines->AddStrings(logdlg->log);
   logdlg->Viewer->SelStart = logdlg->Viewer->Text.Length();
   logdlg->Viewer->Perform(EM_SCROLLCARET, 0, 0);
   logdlg->ShowModal();
   delete logdlg;
}
//---------------------------------------------------------------------------

void __fastcall ViewNNTPLog(TForm *Owner)
{
   TLogDlg *logdlg = new TLogDlg(Owner);
   logdlg->Caption = "NNTP Log";
   //DB->LoadLog(SMTPLog, logdlg->log);
   //logdlg->Viewer->Lines->AddStrings(logdlg->log);
   //logdlg->Viewer->SelStart = logdlg->Viewer->Text.Length();
   //logdlg->Viewer->Perform(EM_SCROLLCARET, 0, 0);
   //logdlg->ShowModal();
   //delete logdlg;
}
//---------------------------------------------------------------------------
#endif

__fastcall TLogDlg::TLogDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Viewer->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "logdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Viewer->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   delete ini;
   log = new TStringList();
}
//---------------------------------------------------------------------------

__fastcall TLogDlg::~TLogDlg()
{
   if (ModalResult == mrOk && IntToStr(Width)+IntToStr(Height) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "logdlg", ini);
      delete ini;
   }
   delete log;
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::Copy1Click(TObject *Sender)
{
   Viewer->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::PopupMenu1Popup(TObject *Sender)
{
   Copy1->Enabled = Viewer->SelLength > 0;
   Selectall1->Enabled = Viewer->Text.Length() > 0;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
   Clear1->Enabled = Viewer->Text.Length() > 0;
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::ChainsBoxClick(TObject *Sender)
{
   if (ChainsBox->Checked)
   {
      Viewer->Lines->Clear();
      Snooze(200);
      Viewer->Lines->BeginUpdate();
	  int i = 0; while (i < log->Count)
	  {
         AnsiString line = log->Strings[i];

         if (line.Pos("Session:") == 1)
         {
            Viewer->Lines->Add(line);
         }

         if (line.Pos("Mixmaster reports") > 0)
		 {
             while (i < log->Count - 1)
             {
				line = log->Strings[++i].Trim();
				Viewer->Lines->Add(line);
				if (line.IsEmpty())
				   break;
			 }
		 }
		 i++;
	  }
	  Viewer->SelStart = Viewer->Text.Length() - 2;
	  Viewer->Perform(EM_SCROLLCARET, 0, 0);
	  Viewer->Lines->EndUpdate();
	  ActiveControl = Viewer;
   }
   else
   {
	  Viewer->Lines->Clear();
	  Snooze(200);
	  Viewer->Lines->BeginUpdate();
	  Viewer->Lines->AddStrings(log);
	  Viewer->SelStart = Viewer->Text.Length();
	  Viewer->Perform(EM_SCROLLCARET, 0, 0);
	  Viewer->Lines->EndUpdate();
	  ActiveControl = Viewer;
   }
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::Selectall1Click(TObject *Sender)
{
   Viewer->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TLogDlg::Clear1Click(TObject *Sender)
{
   if (MsgBox(this,
	  "Clear the SMTP log now?   ",
	   MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
		  return;

   Viewer->Clear();
   log->Clear();
   DB->SaveLog(SMTPLog, log);
}
//---------------------------------------------------------------------------

