/*
*  optionsdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "optionsdlgs.h"
#include "composewins.h"
#include "utility.h"
#include "qsdbs.h"
//#include "proxydlgs.h"
#include "mixmaster.h"
#include "global.h"
#include "gpg.h"
#include "help.h"
#include "pgpcache.h"
#include "eraser.h"
#include "proxymandlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TOptionsDlg *OptionsDlg;
//---------------------------------------------------------------------------

bool __fastcall EditOptions(TForm *Owner, int page)
{
   assert(page >= 0);
   assert(page < 6);
   TOptionsDlg *optionsdlg = new TOptionsDlg(Owner);
   optionsdlg->PageControl->ActivePageIndex = page;
   optionsdlg->ShowModal();
   bool ok = optionsdlg->saved;
   bool notify = optionsdlg->notify;
   delete optionsdlg;
   if (notify)
	  Application->MainForm->Perform(WM_APPINICHANGE, 0, 0);
   return ok;
}
//---------------------------------------------------------------------------

/*bool __fastcall EditOptionsOnly(TForm *Owner, int page)
{
   assert(page >= 0);
   assert(page < 6);
   TOptionsDlg *optionsdlg = new TOptionsDlg(Owner);
   optionsdlg->PageControl->ActivePageIndex = page;
   for (int i = 0; i < optionsdlg->PageControl->PageCount; i++)
   {
	  if (i != page)
		 optionsdlg->PageControl->Pages[i]->TabVisible = false;
   }
   //if (OptionsDlg->SMTPTab->TabVisible)
   //   OptionsDlg->smtpmode = true;
   optionsdlg->ShowModal();
   bool ok = optionsdlg->saved;
   delete optionsdlg;
   return ok;
}*/
//---------------------------------------------------------------------------

bool __fastcall ConfigureMIXandHTTP(TForm *Owner)
{
   TOptionsDlg *optionsdlg = new TOptionsDlg(Owner);
   optionsdlg->GeneralTab->TabVisible = false;
   //OptionsDlg->SMTPTab->TabVisible = false;
   optionsdlg->FontTab->TabVisible = false;
   optionsdlg->PGPTab->TabVisible = false;
   optionsdlg->ERASTab->TabVisible = false;
   optionsdlg->STUNTab->TabVisible = false;
   optionsdlg->PageControl->ActivePage = optionsdlg->MixmasterTab;
   optionsdlg->ShowModal();
   bool ok = optionsdlg->saved;
   delete optionsdlg;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall urlpath(const AnsiString &url)
{
   int pos, len = url.Length();
   for (int i = len; i > 1; i--)
   {
	  if (url[i] == '/')
	  {
		 pos = i;
		 break;
	  }
   }
   return url.SubString(1, pos);
}
//---------------------------------------------------------------------------

__fastcall TOptionsDlg::TOptionsDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   mix  = new TStringList();
   pgp  = new TStringList();
   http = new TStringList();
   gen  = new TStringList();
   eras = new TStringList();
   seckeys = new TStringList();
   seckeys->Sorted = true;
//TStringList *tmp = new TStringList();
//DB->GetText("urls", tmp);
//look(tmp);
//delete tmp;
   TIniFile *ini = new TIniFile(AppIni());
   ini->ReadSectionValues("settings", gen);
   AssignFont(EdiFont->Font, ReadString(gen, "editorfont", OptsDefEditorFont));
   AssignFont(DlgFont->Font, ReadString(gen, "dialogfont", OptsDefDialogFont));
   AssignFont(WinFont->Font, ReadString(gen, "windowfont", OptsDefWindowFont));
   DB->GetText("urls"+DBPS+allpingers, MIXAllPingersTxt->Items);
   if (!MIXAllPingersTxt->Items->Count)
	  MIXAllPingersTxt->Items->CommaText = ReadString(gen, "allpingersurls", MixAllPingersURLs);

   MIXMixTls->Items->Clear();
   MIXMixTls->Items->Add("use same server/directory as allpingers.txt");
   MIXMixTls->ItemIndex = 0;

   //for (int i = 0; i < MIXAllPingersTxt->Items->Count; i++)
   //{
   //	  MIXMixTls->Items->Add(urlpath(MIXAllPingersTxt->Items->Strings[i])+"mix.tls");
   //}

   //DB->GetText("urls"+DBPS+mixmixtls, MIXMixTls->Items);
   //if (!MIXMixTls->Items->Count)
   //	  MIXMixTls->Items->CommaText = MixMixMxURL;
   MIXAfterTime->Font->Assign(WinFont->Font);
   //NYMNewWindow->Visible = FileExists(AppDir()+"qsa.exe");
   delete ini;

   SetWindowFont();
   Font->Assign(DlgFont->Font);
   EdiFont->Caption = EdiFont->Font->Name + "  " + IntToStr(EdiFont->Font->Size) + "pt";
   DlgFont->Caption = DlgFont->Font->Name + "  " + IntToStr(DlgFont->Font->Size) + "pt";
   WinFont->Caption = WinFont->Font->Name + "  " + IntToStr(WinFont->Font->Size) + "pt";
   SetControlPositions();

   //STUNTab->TabVisible = false;
   DB->LoadConfig(HTTPConfig, http);
   DB->LoadConfig(PGPConfig, pgp);
   DB->LoadConfig(MIXConfig, mix);
   DB->LoadConfig(ERASConfig, eras);
   Load();

   CheckDisableProxySelectors();
   MIXUpdateClick(MIXUpdate);
   MIXDirectClick(MIXDirect);
   PNGUpdateClick(PNGUpdate);
   PGPBoxClick(PGPBox);
   ERASBoxClick(ERASBox);
   MIXUpdateFreqChange(NULL);
   MIXAfterBoxClick(NULL);
}
//---------------------------------------------------------------------------

__fastcall TOptionsDlg::~TOptionsDlg()
{
   delete seckeys;
   delete eras;
   delete stun;
   delete gen;
   delete http;
   delete pgp;
   delete mix;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::FormCloseQuery(TObject *Sender,
	  bool &CanClose)
{
   //if (MIXAllPingersTxt->Text.IsEmpty())
   //   MIXAllPingersTxt->Text = MixAllPingersURL;
   if (MIXAllPingersTxt->ItemIndex < 0)
      MIXAllPingersTxt->ItemIndex = 0;

   if (ModalResult == mrOk)
   {
	  if (PGPBox->Checked)
	  {
		 AnsiString secring = PGPSecring->Text;
		 if (secring.IsEmpty())
		 {
			MsgBox(this,
			   "Problem! You have PGP Public Key Encryption enabled, but You   \n"
			   "haven't specified your PGP private keyring. You'll have to  \n"
			   "fix that.",
			   MT_INFO, MB_OK);

			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPSecring;
			CanClose = false;
			return;
		 }

		 if (!FileExists(secring))
		 {
			MsgBox(this,
			   "Problem! The file, " + secring + ", doesn't exist!   \n"
			   "You'll need to fix it. If you aren't sure leave it blank.",
				MT_INFO, MB_OK);
			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPSecring;
			CanClose = false;
			return;
		 }

		 AnsiString pubring = PGPPubring->Text;
		 if (pubring.IsEmpty())
		 {
			MsgBox(this,
			   "Problem! You have PGP Public Key Encryption enabled, but You   \n"
			   "haven't specified your PGP public keyring. You'll have to  \n"
			   "fix that.",
			   MT_INFO, MB_OK);

			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPPubring;
			CanClose = false;
			return;
		 }

		 if (!FileExists(pubring))
		 {
			MsgBox(this,
			   "Problem! The file, " + pubring + ", doesn't exist!  \n"
			   "You'll need to fix it. If you aren't sure leave it blank.",
				MT_INFO, MB_OK);
			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPPubring;
			CanClose = false;
			return;
		 }

		 if (!secring.IsEmpty() && !pubring.IsEmpty() &&
			GetShortName(ExtractFilePath(secring)).LowerCase() !=
			GetShortName(ExtractFilePath(pubring)).LowerCase())
		 {
			MsgBox(this,
			   "Problem! Your PGP private keyring and your PGP secret keyring   \n"
			   "aren't in the same directory. Put them both in the same place.  \n",
				MT_INFO, MB_OK);
			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPSecring;
			CanClose = false;
			return;
		 }

		 if (GetShortName(pubring).LowerCase() == GetShortName(secring).LowerCase())
		 {
			MsgBox(this,
			   "Problem! Your PGP secret keyring and your PGP public keyring   \n"
			   "are the same file. You can do that!",
				MT_INFO, MB_OK);
			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPSecring;
			CanClose = false;
			return;
		 }

		 if (PGPDefKey->ItemIndex < 0)
		 {
			MsgBox(this,
			   "Problem! You haven't selected a default PGP key. You'll need   \n"
			   "to fix that.",
				MT_INFO, MB_OK);
			PageControl->ActivePage = PGPTab;
			ActiveControl = PGPDefKey;
			CanClose = false;
		 }
	  }
   }

   if (Modified())
   {
	  if (ModalResult == mrCancel)
	  {
		 switch (MsgBox(this,
		   "Your options are modified. Save your changes? ",
			MT_CONFIRM, MB_YESNOCANCEL))
		 {
			case IDCANCEL:
			   CanClose = false;
			case IDNO:
			   return;
		 }
	  }

	  ModalResult = mrOk;
	  Save();
   }
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::Modified()
{
   return GENModified() || HTTPModified() || PGPModified() || MIXModified() ||
	  ERASModified();
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::PGPModified()
{
   return PGPBox->Checked != ReadBool(pgp, "pgp", false) ||
	  PGPSecring->Text != ReadString(pgp, "secring", "") ||
	  PGPPubring->Text != ReadString(pgp, "pubring", "") ||
	  PGPDefKey->Text != ReadString(pgp, "defkey", "") ||
	  PGPEncToDef->Checked != ReadBool(pgp, "encdef", false) ||
	  PGPMaskVer->Checked != ReadBool(pgp, "maskver", true) ||
	  PGPCacheUpDown->Position != ReadInteger(pgp, GPGcacheminutes, GPGCacheMinutes);
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::GENModified()
{
   AnsiString marks = ReadString(gen, "quotemarks", OptsDefQuoteMarks);
   return GENWrapUpDown->Position != ReadInteger(gen, "wordwraponsend", OptsDefMessageWrap) ||
	  GENQuoteChars->Text != DropQuotes(marks) ||
	  //GENLoadNew->Checked != ReadBool(gen, "loadnew", true) ||
	  GENGridLines->Checked != ReadBool(gen, "gridlines", false) ||
	  GENOpenMessage->ItemIndex != ReadInteger(gen, "newmessage", 1) ||
	  GENUserMode->ItemIndex != ReadInteger(gen, "expertmode", 1) ||
      GetFontStr(DlgFont->Font) != ReadString(gen, "dialogfont", OptsDefDialogFont) ||
	  GetFontStr(WinFont->Font) != ReadString(gen, "windowfont", OptsDefWindowFont) ||
      GetFontStr(EdiFont->Font) != ReadString(gen, "editorfont", OptsDefEditorFont) ||
      GENHideCompWinBox->Checked != ReadBool(gen, "composeproxyselector", false) ||
      GENHideStatsDlgBox->Checked != ReadBool(gen, "statsproxyselector", false) ||
	  GENMaxLogUpDown->Position != ReadInteger(gen, "maxsmtplog", OptsMaxSMTPLogLines) ||
	  GENUseSSLCert->Checked != ReadBool(gen, "usesslcert", false) ||
	  GENSubRandom->Checked != ReadBool(gen, "subrandomdate", false) ||
	  GENCloseSmtp->Checked != ReadBool(gen, "closesmtp", false) ||
	  NYMNewWindow->Checked != ReadBool(gen, "nymnewwindow", true);
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::HTTPModified()
{
   return HTTPTimeoutUpDown->Position != ReadInteger(http, "timeout", 30);
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::MIXModified()
{
   return MIXAllPingersTxt->Text != ReadString(mix, mixallpingers, "") ||
	  //MIXMixTls->Text != ReadString(mix, mixmixtls, "") ||
	  MIXUpdate->Checked != ReadBool(mix, "updateonsend", true) ||
	  MIXUpdateFreq->ItemIndex != ReadInteger(mix, "updatefreq", 0) ||
	  MIXAfterBox->Checked != ReadBool(mix, "updateafter", false) ||
	  MIXAfterTime->Text != ReadString(mix, "aftertime", "08:00") ||
	  PNGUpdate->Checked != ReadBool(mix, "pngupdate", true) ||
	  PNGUpdateFreq->ItemIndex != ReadInteger(mix, "pngupdatefreq", 1) ||
	  MIXMlistTxt->Checked != ReadBool(mix, "mlist.txt", true) ||
	  MIXPubringMix->Checked != ReadBool(mix, "pubring.mix", true) ||
	  MIXRlistTxt->Checked != ReadBool(mix, "rlist.txt", true) ||
	  MIXPubringAsc->Checked != ReadBool(mix, "pubring.asc", true) ||
	  //MIXDummy->Checked != MixGetMixCfgDummies() ||
	  MIXDirect->Checked != ReadBool(mix, "mxdirect", false) ||
	  MIXTlsPreferred->Checked != ReadBool(mix, "tlspreferred", false) ||
	  MIXMixMx->Checked != ReadBool(mix, mixmixmx, true);
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::ERASModified()
{
   return ERASBox->Checked != ReadBool(eras, ERASERenabled, ERASEREnabled) ||
	  ERASGetWipeMethod() != ReadString(eras, ERASERwipemethod, ERASERWipeMethod) ||
	  ERASUpDown->Position != ReadInteger(eras, ERASERpasses, ERASERPasses);
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::CheckDisableProxySelectors()
{
   TStringList *HTTP = new TStringList();
   TStringList *SMTP = new TStringList();
   DB->GetProxyNames(HTTP, SMTP, NULL, NULL);
   GENHideCompWinBox->Enabled = SMTP->Count;
   if (!GENHideCompWinBox->Enabled)
	  GENHideCompWinBox->Checked = false;
   GENHideStatsDlgBox->Enabled = HTTP->Count;
   if (!GENHideStatsDlgBox->Enabled)
	  GENHideStatsDlgBox->Checked = false;
   delete HTTP;
   delete SMTP;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::Load()
{
   GENUserMode->ItemIndex = ReadBool(gen, "expertmode", 1);
   if (GENUserMode->ItemIndex < 0 || GENUserMode->ItemIndex > 1)
	   GENUserMode->ItemIndex = 0;
   SetExpertMode(GENUserMode->ItemIndex);  // 0=novice 1=expert
   GENGridLines->Checked = ReadBool(gen, "gridlines", false);
   GENWrapUpDown->Position = ReadInteger(gen, "wordwraponsend", OptsDefMessageWrap);
   AnsiString marks = ReadString(gen, "quotemarks", OptsDefQuoteMarks);
   GENQuoteChars->Text = DropQuotes(marks);
   GENQuoteCharsChange(NULL);
   GENOpenMessage->ItemIndex = ReadInteger(gen, "newmessage", 1);
   if (GENOpenMessage->ItemIndex < 0 || GENOpenMessage->ItemIndex > 1)
	   GENOpenMessage->ItemIndex = 1;
   //GENUserMode->ItemIndex = ReadInteger(gen, "expertmode", 0);
   GENHideCompWinBox->Checked = ReadBool(gen, "composeproxyselector", false);
   GENHideStatsDlgBox->Checked = ReadBool(gen, "statsproxyselector", false);
   GENMaxLogUpDown->Position = ReadInteger(gen, "maxsmtplog", OptsMaxSMTPLogLines);
   GENUseSSLCert->Checked = ReadBool(gen, "usesslcert", false);
   GENSubRandom->Checked = ReadBool(gen, "subrandomdate", false);
   GENCloseSmtp->Checked = ReadBool(gen, "closesmtp", false);
   HTTPTimeoutUpDown->Position = ReadInteger(http, "timeout", 30);
   NYMNewWindow->Checked = ReadBool(gen, "nymnewwindow", true);

   PGPBox->Checked = ReadBool(pgp, "pgp", false);
   PGPSecring->Text = ReadString(pgp, "secring", "");
   if (FileExists(PGPSecring->Text))
	  PGPLoadKeys();
   PGPPubring->Text = ReadString(pgp, "pubring", "");
   PGPDefKey->ItemIndex = PGPDefKey->Items->IndexOf(ReadString(pgp, "defkey", ""));
   PGPEncToDef->Checked = ReadBool(pgp, "encdef", false);
   PGPMaskVer->Checked = ReadBool(pgp, "maskver", true);
   PGPCacheUpDown->Position = ReadInteger(pgp, GPGcacheminutes, GPGCacheMinutes);

   //if (ReadString(mix, mixallpingers, "").IsEmpty())
   //{
   //	  mix->Values[mixallpingers] = MIXMixTls->Items->Strings[0];
   //	  DB->SaveConfig(MIXConfig, mix);
   //}
   MIXAllPingersTxt->ItemIndex =
	  MIXAllPingersTxt->Items->IndexOf(ReadString(mix, mixallpingers, ""));
   if (MIXAllPingersTxt->ItemIndex < 0)
	  MIXAllPingersTxt->ItemIndex = 0;

   /*if (ReadString(mix, mixmixtls, "").IsEmpty())  // sets value for the first time through
   {
	  mix->Values[mixmixtls] = MIXMixTls->Items->Strings[0];
	  DB->SaveConfig(MIXConfig, mix);
   }
   MIXMixTls->ItemIndex =
	  MIXMixTls->Items->IndexOf(ReadString(mix, mixmixtls, ""));
   if (MIXMixTls->ItemIndex < 0)
	  MIXMixTls->ItemIndex = 0;
   */

   MIXMixMx->Checked = ReadBool(mix, mixmixmx, true);
   MIXUpdate->Checked = ReadBool(mix, "updateonsend", true);
   MIXUpdateFreq->ItemIndex = ReadInteger(mix, "updatefreq", 0);
   if (MIXUpdateFreq->ItemIndex < 0)
	   MIXUpdateFreq->ItemIndex = 0;
   MIXAfterBox->Checked = ReadBool(mix, "updateafter", false);
   MIXAfterTime->ItemIndex = MIXAfterTime->Items->IndexOf(ReadString(mix, "aftertime", "08:00"));
   if (MIXAfterTime->ItemIndex < 0)
	   MIXAfterTime->ItemIndex = 8;
   MIXAfterBox->Enabled = MIXUpdate->Checked && MIXUpdateFreq->ItemIndex == 1;
   MIXAfterTime->Enabled = MIXUpdate->Checked && MIXAfterBox->Checked;
   PNGUpdate->Checked = ReadBool(mix, "pngupdate", true);
   PNGUpdateFreq->ItemIndex = ReadBool(mix, "pngupdatefreq", 1);
   MIXMlistTxt->Checked = ReadBool(mix, "mlist.txt", true);
   MIXPubringMix->Checked = ReadBool(mix, "pubring.mix", true);
   MIXRlistTxt->Checked = ReadBool(mix, "rlist.txt", true);
   MIXPubringAsc->Checked = ReadBool(mix, "pubring.asc", true);
   //MIXDummy->Checked = MixGetMixCfgDummies();
   MIXDirect->Checked = ReadBool(mix, "mxdirect", false);
   MIXTlsPreferred->Checked = ReadBool(mix, "tlspreferred", false);

   ERASBox->Checked = ReadBool(eras, ERASERenabled, ERASEREnabled);
   ERASSetWipeMethod(ReadString(eras, ERASERwipemethod, ERASERWipeMethod));
   ERASUpDown->Position = ReadInteger(eras, ERASERpasses, ERASERPasses);
   ERASBox->Checked = ReadBool(eras, ERASERenabled, ERASEREnabled);
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::Save()
{
   if (GENModified())
   {
	  AnsiString fontstr;
	  gen->Values["wordwraponsend"] = GENWrapUpDown->Position;
	  gen->Values["quotemarks"] = "\"\"" + GENQuoteChars->Text + "\"\"";
	  gen->Values["gridlines"] = (int)GENGridLines->Checked;
	  gen->Values["newmessage"] = GENOpenMessage->ItemIndex;
	  gen->Values["usesslcert"] = (int)GENUseSSLCert->Checked;
	  gen->Values["subrandomdate"] = (int)GENSubRandom->Checked;
	  gen->Values["maxsmtplog"] = GENMaxLogUpDown->Position;
	  gen->Values["closesmtp"] = (int)GENCloseSmtp->Checked;
	  gen->Values["nymnewwindow"] = (int)NYMNewWindow->Checked;

	  TIniFile *ini = new TIniFile(AppIni());
	  ini->WriteInteger("settings", "wordwraponsend", GENWrapUpDown->Position);
	  ini->WriteString("settings", "quotemarks", "\"" + GENQuoteChars->Text + "\"");
	  ini->WriteBool("settings", "gridlines", GENGridLines->Checked);
	  ini->WriteInteger("settings", "newmessage", GENOpenMessage->ItemIndex);
	  ini->WriteBool("settings", "usesslcert", GENUseSSLCert->Checked);
	  ini->WriteBool("settings", "subrandomdate", GENSubRandom->Checked);
	  ini->WriteInteger("settings", "maxsmtplog", GENMaxLogUpDown->Position);
	  ini->WriteBool("settings", "closesmtp", GENCloseSmtp->Checked);
	  ini->WriteBool("settings", "nymnewwindow", NYMNewWindow->Checked);

	  // handle option that might change the compose windows
	  notify = false;
	  if (GENUserMode->ItemIndex != ReadInteger(gen, "expertmode", 1))
	  {
		 int expert = GENUserMode->ItemIndex;
		 ini->WriteInteger("settings", "expertmode", expert);
		 gen->Values["expertmode"] = expert;
		 notify = true;
	  }
	  fontstr = GetFontStr(DlgFont->Font);
	  if (fontstr != ReadString(gen, "dialogfont", OptsDefDialogFont))
	  {
		 ini->WriteString("settings", "dialogfont", fontstr);
		 gen->Values["dialogfont"] = fontstr;
		 notify = true;
	  }
	  fontstr = GetFontStr(EdiFont->Font);
	  if (fontstr != ReadString(gen, "editorfont", OptsDefEditorFont))
	  {
		 ini->WriteString("settings", "editorfont", fontstr);
		 gen->Values["editorfont"] = fontstr;
		 notify = true;
	  }
	  fontstr = GetFontStr(WinFont->Font);
	  if (fontstr != ReadString(gen, "windowfont", OptsDefWindowFont))
	  {
		 ini->WriteString("settings", "windowfont", fontstr);
		 gen->Values["windowfont"] = fontstr;
		 notify = true;
	  }
	  if (GENHideCompWinBox->Checked != ReadBool(gen, "composeproxyselector", false))
	  {
		 ini->WriteBool("settings", "composeproxyselector", GENHideCompWinBox->Checked);
		 gen->Values["composeproxyselector"] = (int)GENHideCompWinBox->Checked;
		 notify = true;
	  }
	  if (GENHideStatsDlgBox->Checked != ReadBool(gen, "statsproxyselector", false))
	  {
		 ini->WriteBool("settings", "statsproxyselector", GENHideStatsDlgBox->Checked);
		 gen->Values["statsproxyselector"] = (int)GENHideStatsDlgBox->Checked;
		 notify = true;
	  }
	  //if (GENMaxLogUpDown->Position != ReadInteger(gen, "maxsmtplog", OptsMaxSMTPLogLines))
	  //{
	  //   ini->WriteInteger("settings", "maxsmtplog", GENMaxLogUpDown->Position);
	  //   gen->Values["maxsmtplog"] = GENMaxLogUpDown->Position;
	  //}
	  delete ini;
   }

   if (HTTPModified())
   {
	  http->Clear();
	  http->Values["timeout"] = HTTPTimeoutUpDown->Position;
	  DB->SaveConfig(HTTPConfig, http);
   }

   if (PGPModified())
   {
	  if (PGPBox->Checked != ReadBool(pgp, "pgp", false))
		 notify = true;
	  pgp->Values["pgp"] = (int)PGPBox->Checked;
	  pgp->Values["secring"] = PGPSecring->Text;
	  pgp->Values["pubring"] = PGPPubring->Text;
	  pgp->Values["defkey"] = PGPDefKey->Text;
	  pgp->Values["encdef"] = (int)PGPEncToDef->Checked;
	  pgp->Values["maskver"] = (int)PGPMaskVer->Checked;
	  pgp->Values[GPGcacheminutes] = PGPCacheUpDown->Position;
	  DB->SaveConfig(PGPConfig, pgp);
	  SetGPGConfValues();
   }

   if (MIXModified())
   {
	  mix->Values[mixallpingers] = MIXAllPingersTxt->Text;
	  mix->Values[mixmixtls] = urlpath(MIXAllPingersTxt->Text)+"mix.tls";
	  mix->Values[mixmixmx] = (int)MIXMixMx->Checked;
	  mix->Values["updateonsend"] = (int)MIXUpdate->Checked;
	  mix->Values["updatefreq"] = MIXUpdateFreq->ItemIndex;
	  mix->Values["mlist.txt"] = (int)MIXMlistTxt->Checked;
	  mix->Values["pubring.mix"] = (int)MIXPubringMix->Checked;
	  mix->Values["rlist.txt"] = (int)MIXRlistTxt->Checked;
	  mix->Values["pubring.asc"] = (int)MIXPubringAsc->Checked;
	  mix->Values["mxdirect"] = (int)MIXDirect->Checked;
	  mix->Values["tlspreferred"] = (int)MIXTlsPreferred->Checked;
	  //if (MIXDummy->Checked != MixGetMixCfgDummies())
	  //MixSetMixCfgDummies(MIXDummy->Checked);
	  mix->Values["updateafter"] = (int)MIXAfterBox->Checked;
	  mix->Values["aftertime"] = MIXAfterTime->Text;
	  mix->Values["pngupdate"] = (int)PNGUpdate->Checked;
	  mix->Values["pngupdatefreq"] = PNGUpdateFreq->ItemIndex;
	  DB->SaveConfig(MIXConfig, mix);
   }

   if (ERASModified())
   {
	  eras->Values[ERASERenabled] = (int)ERASBox->Checked;
	  eras->Values[ERASERwipemethod] = ERASGetWipeMethod();
	  eras->Values[ERASERpasses] = ERASUpDown->Position;
	  DB->SaveConfig(ERASConfig, eras);
   }

   saved = true;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::ERASSetWipeMethod(const AnsiString &method)
{
   if (method == "dod") ERASDodWipe->Checked = true;
   else if (method == "gut") ERASGutWipe->Checked = true;
   else ERASRndWipe->Checked = true;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TOptionsDlg::ERASGetWipeMethod()
{
   if (ERASDodWipe->Checked) return "dod";
   if (ERASGutWipe->Checked) return "gut";
   return "rnd";
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::SetWindowFont()
{
   GENWrap->Font->Assign(WinFont->Font);
   GENQuoteChars->Font->Assign(WinFont->Font);
   GENMaxLog->Font->Assign(WinFont->Font);
   HTTPTimeout->Font->Assign(WinFont->Font);
   PGPSecring->Font->Assign(WinFont->Font);
   PGPPubring->Font->Assign(WinFont->Font);
   PGPDefKey->Font->Assign(WinFont->Font);
   PGPCacheMins->Font->Assign(WinFont->Font);
   MIXAllPingersTxt->Font->Assign(WinFont->Font);
   MIXUpdateFreq->Font->Assign(WinFont->Font);
   ERASPasses->Font->Assign(WinFont->Font);
}
//---------------------------------------------------------------------------
// deprecated
void __fastcall TOptionsDlg::MIXResetBtnClick(TObject *Sender)
{
   if (MsgBox(this,
	  "Reset the allpingers URL to original? ",
	   MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  MIXAllPingersTxt->Clear();
	  Snooze(200);
	  //MIXAllPingersTxt->Text = MixAllPingersURL;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::SetExpertMode(bool mode)
{
   Label7->Visible = mode;

   GENQuoteChars->Visible = mode;
   GENGridLines->Visible = mode;
   GENOpenMessage->Visible = mode;
   GENHideSelectors->Visible = mode;
   if (!mode)
   {
	  GENHideStatsDlgBox->Checked = false;
	  GENHideCompWinBox->Checked = false;
   }
   GENMaxLog->Visible = mode;
   GENMaxLogUpDown->Visible = mode;
   GENSubRandom->Visible = mode;
   GENCloseSmtp->Visible = mode;
   //NYMNewWindow->Visible = mode;
   Label20->Visible = mode;
   Label21->Visible = mode;
   //HTTPProxyBtn->Caption = mode ? "&Proxies" : "&Proxy";
   MIXMixMx->Visible = mode;
   MIXUpdate->Visible = mode;
   MIXUpdateFreq->Visible = mode;
   PNGUpdate->Visible = mode;
   PNGUpdateFreq->Visible = mode;
   //MIXDummy->Visible = mode;
   MIXDirect->Visible = mode;
   MIXTlsPreferred->Visible = mode;
   MIXMlistTxt->Visible = mode;
   MIXPubringMix->Visible = mode;
   MIXRlistTxt->Visible = mode;
   MIXPubringAsc->Visible = mode;
   MIXAfterBox->Visible = mode;
   MIXAfterTime->Visible = mode;
   PGPTab->TabVisible = mode;
   //STUNTab->TabVisible = mode;
   ERASTab->TabVisible = mode;
   expertmode = mode;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::HTTPProxyBtnClick(TObject *Sender)
{
   AnsiString name;
   bool ok; // = false;
   //if (GENUserMode->ItemIndex) // expertmode == 1 else 0
   ok = ManageProxies(this);
   //else
   //   name = EditHTTPProxy(this);

   if (ok)// || !name.IsEmpty())
      CheckDisableProxySelectors();
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::SMTPProxyBtnClick(TObject *Sender)
{
   //EditSMTPProxy(this);
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::MIXUpdateClick(TObject *Sender)
{
   if (MIXUpdate->Checked)
   {
	  MIXUpdateFreq->Enabled = true;
	  //MIXUpdateFreq->Color = clWindow;
	  if (MIXUpdateFreq->ItemIndex == 1)
	  {
		 MIXAfterBox->Enabled = true;
		 MIXAfterTime->Enabled = MIXAfterBox->Checked;
	  }
	  Label1->Enabled = true;
	  Label23->Enabled = true;
	  MIXAllPingersTxt->Enabled = true;
	  MIXMixTls->Enabled = true;
	  PNGUpdate->Enabled = true;
	  PNGUpdateFreq->Enabled = PNGUpdate->Checked;
	  MIXMixMx->Enabled = PNGUpdate->Checked;
   }
   else
   {
	  MIXUpdateFreq->Enabled = false;
	  //MIXUpdateFreq->ParentColor = true;
	  MIXAfterBox->Enabled = false;
	  MIXAfterTime->Enabled = false;

	  Label1->Enabled = false;
	  Label23->Enabled = false;
	  MIXAllPingersTxt->Enabled = false;
	  MIXMixTls->Enabled = false;
	  PNGUpdate->Enabled = false;
	  PNGUpdateFreq->Enabled = false;
	  MIXMixMx->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::SetControlPositions()
{
   int l = Canvas->TextWidth(MIXUpdate->Caption);
   MIXUpdateFreq->Left = MIXUpdate->Left + l + 12;
   l = Canvas->TextWidth(ERASRndWipe->Caption);
   ERASPasses->Left = ERASRndWipe->Left + l + 15;
   ERASUpDown->Left = ERASPasses->Left + ERASPasses->Width;
   Label19->Left = ERASUpDown->Left + ERASUpDown->Width + 8;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::DialogFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(DlgFont->Font);
   if (FontDialog->Execute())
   {
	  DlgFont->Font->Assign(FontDialog->Font);
      Font->Assign(FontDialog->Font);
	  DlgFont->Caption = Font->Name + "  " +
         IntToStr(Font->Size) + "pt";
      SetControlPositions();
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::EditorFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(EdiFont->Font);
   if (FontDialog->Execute())
   {
      EdiFont->Font->Assign(FontDialog->Font);
      EdiFont->Caption = EdiFont->Font->Name + "  " +
         IntToStr(EdiFont->Font->Size) + "pt";
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::WindowFontBtnClick(TObject *Sender)
{
   FontDialog->Font->Assign(WinFont->Font);
   if (FontDialog->Execute())
   {
      WinFont->Font->Assign(FontDialog->Font);
      WinFont->Caption = WinFont->Font->Name + "  " +
         IntToStr(WinFont->Font->Size) + "pt";
	  SetWindowFont();
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::GENUserModeClick(TObject *Sender)
{
   if (expertmode && GENUserMode->ItemIndex == 0)
   {
      int x = DB->QueueCount(MailQueue);
      int y = DB->QueueCount(ProbQueue);

      if (x && y)
      {
         MsgBox(this,
           "Sorry, you have " + IntToStr(x) + " message(s) in the Mail Queue and   \n" +
           IntToStr(y) + " message(s) in the Problem Queue. You need to clear the   \n"
           "queues before you can select Whistle Blower mode.",
            MT_INFO, MB_OK);
         GENUserMode->ItemIndex = 1;
         return;
      }
      if (x)
      {
         MsgBox(this,
           "Sorry, you have " + IntToStr(x) + " message(s) in the Mail Queue. You  \n" +
           "need to clear the queue before you can select Whistle   \n"
           "Blower mode.",
            MT_INFO, MB_OK);
         GENUserMode->ItemIndex = 1;
         return;
      }
	  if (x)
      {
         MsgBox(this,
           "Sorry, you have " + IntToStr(y) + " message(s) in the Problem Queue. You  \n" +
		   "need to clear the queue before you can select Whistle   \n"
		   "Blower mode.",
            MT_INFO, MB_OK);
         GENUserMode->ItemIndex = 1;
         return;
      }
   }
   SetExpertMode(GENUserMode->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPDefKeyDrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
   AnsiString line = PGPDefKey->Items->Strings[Index];
   //line = GetTok(1, line, ':') + " " + GetTok(0, line, ':');
   PGPDefKey->Canvas->TextRect(Rect, Rect.Left, Rect.Top - 1, line);
}
//---------------------------------------------------------------------------

int __fastcall TOptionsDlg::PGPLoadKeys()
{
   assert(FileExists(PGPSecring->Text));
   AnsiString defkey = PGPDefKey->Text;
   seckeys->Clear();

   AnsiString key, keyid;
   GPGListSecretKeysWithColonsRingName(PGPSecring->Text, seckeys);

   TStringList *keys = new TStringList();
   for (int i = 0; i < seckeys->Count; i++)
   {
      key = seckeys->Strings[i];
      if (key.Pos("sec") != 1)
         continue;

	  keyid = GetTok(4, key, ':');
      keys->Add("0x" + keyid.SubString(keyid.Length() - 7, 8));
   }

   if (keys->Text != PGPDefKey->Items->Text)
   {
      PGPDefKey->Items->Text = keys->Text;
      PGPDefKey->ItemIndex = PGPDefKey->Items->IndexOf(defkey);
   }
   delete keys;

   return PGPDefKey->Items->Count;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPSecringExit(TObject *Sender)
{
   /*if (PGPSecring->Text.IsEmpty())
   {
	  MsgBox(this,
         "Problem! You haven't specified your PGP secret keyring!  ",
         MT_INFO, MB_OK);
      PageControl->ActivePage = PGPTab;
      ActiveControl = PGPSecring;
	  return;
   }*/
   bool ok = FileExists(PGPSecring->Text);
   if (!ok && !PGPSecring->Text.IsEmpty())
   {
	  MsgBox(this,
		 "Problem! Your PGP secret key ring doesn't exist!  ",
		 MT_INFO, MB_OK);
	  PageControl->ActivePage = PGPTab;
	  ActiveControl = PGPSecring;
	  return;
   }
   if (ok)
	  PGPLoadKeys();
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPPubringExit(TObject *Sender)
{
   /*if (PGPPubring->Text.IsEmpty())
   {
      MsgBox(this,
         "Problem! You haven't specified your PGP public keyring!  ",
         MT_INFO, MB_OK);
      PageControl->ActivePage = PGPTab;
      ActiveControl = PGPPubring;
      return;
   }*/
   if (!PGPPubring->Text.IsEmpty() && !FileExists(PGPPubring->Text))
   {
      MsgBox(this,
         "Problem! Your PGP public key ring doesn't exist!  ",
         MT_INFO, MB_OK);
      PageControl->ActivePage = PGPTab;
      ActiveControl = PGPPubring;
      return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::BrowseSecBtnClick(TObject *Sender)
{
   AnsiString pubring = PGPPubring->Text;
   if (!pubring.IsEmpty() && FileExists(pubring))
      OpenSecringDialog->InitialDir = ExtractFilePath(pubring);

   if (OpenSecringDialog->Execute())
   {
	  AnsiString secring = OpenSecringDialog->FileName;
      //if (CheckPGPSecringOk(secring))
      {
		 PGPSecring->Text = secring;
		 PGPLoadKeys();
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::BrowsePubBtnClick(TObject *Sender)
{
   AnsiString secring = PGPSecring->Text;
   if (!secring.IsEmpty() && FileExists(secring))
      OpenPubringDialog->InitialDir = ExtractFilePath(secring);

   if (OpenPubringDialog->Execute())
   {
      AnsiString pubring = OpenPubringDialog->FileName;
      //if (CheckPGPPubringOk(pubring))
      {
         PGPPubring->Text = pubring;
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::CheckPGPSecringOk(const AnsiString &Secring)
{
   if (Secring.IsEmpty())
      return true;

   if (!FileExists(Secring))
   {
      MsgBox(this,
         "The file, " + Secring + ", doesn't exist!",
		  MT_INFO, MB_OK);
      PageControl->ActivePage = PGPTab;
      ActiveControl = PGPSecring;
      return false;
   }

   AnsiString pubring = PGPPubring->Text;
   if (!pubring.IsEmpty() && FileExists(pubring))
   {
      if (ExtractFilePath(Secring) != ExtractFilePath(pubring))
      {
         MsgBox(this,
            "This file, " + Secring + ", isn't in the   \n"
            "same directory as your pgp public keyring.  \n"
			"They need to be the same!  ",
             MT_INFO, MB_OK);

         PageControl->ActivePage = PGPTab;
         ActiveControl = PGPSecring;
         return false;
      }
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TOptionsDlg::CheckPGPPubringOk(const AnsiString &Pubring)
{
   if (Pubring.IsEmpty())
      return true;

   if (!FileExists(Pubring))
   {
      MsgBox(this,
         "The file, " + Pubring + ", doesn't exist!",
		  MT_INFO, MB_OK);
      PageControl->ActivePage = PGPTab;
      ActiveControl = PGPPubring;
      return false;
   }

   AnsiString secring = PGPSecring->Text;
   if (!secring.IsEmpty() && FileExists(secring))
   {
      if (ExtractFilePath(Pubring) != ExtractFilePath(secring))
      {
         MsgBox(this,
            "This file, " + Pubring + ", isn't in the   \n"
            "same directory as your pgp private keyring.  \n"
            "They need to be the same!  ",
             MT_INFO, MB_OK);
         return false;
      }
   }

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPDefKeyChange(TObject *Sender)
{
   AnsiString thekey = PGPDefKey->Text;
   if (!thekey.IsEmpty())
   {
      int found = 0;
      for (int i = 0; i < seckeys->Count; i++)
      {
         if (seckeys->Strings[i].Pos(thekey) > 0)
            found++;
      }
      if (found > 1)
	  {
		 Snooze(200);
         MsgBox(this,
            "Warning! The user ID you've chosen appears " + IntToStr(found) + " times  \n"
            "on your secret keyring. If you use this ID, only the first key   \n"
            "on the ring, with this ID, will be used when signing. It is best   \n"
            "your user ID be unique.", MT_WARN, MB_OK);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPBoxClick(TObject *Sender)
{
   if (PGPBox->Checked)
   {
      PGPSecring->Color = clWindow;
      PGPSecring->Enabled = true;
      BrowseSecBtn->Enabled = true;
      PGPPubring->Color = clWindow;
      PGPPubring->Enabled = true;
      BrowsePubBtn->Enabled = true;
	  PGPDefKey->Color = clWindow;
      PGPDefKey->Font->Assign(WinFont->Font); //Color = PGPPubring->Font->Color;
      PGPDefKey->Enabled = true;
      PGPEncToDef->Enabled = true;
      PGPCacheMins->Enabled = true;
      PGPCacheMins->Color = clWindow;
      PGPCacheMins->Font->Assign(WinFont->Font);// = PGPPubring->Font->Color;
      PGPCacheUpDown->Enabled = true;
      PGPCacheClrBtn->Enabled = true;
      PGPMaskVer->Enabled = true;
      Label12->Enabled = true;
      Label14->Enabled = true;
      Label13->Enabled = true;
      Label22->Enabled = true;
   }
   else
   {
      PGPSecring->ParentColor = true;
      PGPSecring->Enabled = false;
	  BrowseSecBtn->Enabled = false;
      PGPPubring->ParentColor = true;
      PGPPubring->Enabled = false;
      BrowsePubBtn->Enabled = false;
      PGPDefKey->Font->Color = clGrayText;
      PGPDefKey->ParentColor = true;
      PGPDefKey->Enabled = false;
      PGPEncToDef->Enabled = false;
      PGPCacheMins->Enabled = false;
      PGPCacheMins->ParentColor = true;
      PGPCacheMins->Font->Color = clGrayText;
      PGPCacheUpDown->Enabled = false;
      PGPCacheClrBtn->Enabled = false;
      PGPMaskVer->Enabled = false;
      Label12->Enabled = false;
      Label14->Enabled = false;
      Label13->Enabled = false;
      Label22->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/options.htm>options");
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPDefKeyDropDown(TObject *Sender)
{
//
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PGPDefKeyEnter(TObject *Sender)
{
   if (!PGPSecring->Text.IsEmpty() && FileExists(PGPSecring->Text))
      PGPLoadKeys();
}
//---------------------------------------------------------------------------


void __fastcall TOptionsDlg::PGPCacheClrBtnClick(TObject *Sender)
{
   assert(PGPCache != NULL);
   PGPCache->Clear();
   MsgBox(this, "Passphrase cache cleared!   ", MT_INFO, MB_OK);
}
//---------------------------------------------------------------------------


void __fastcall TOptionsDlg::ERASBoxClick(TObject *Sender)
{
   if (ERASBox->Checked)
   {
      ERASDodWipe->Enabled = true;
      ERASGutWipe->Enabled = true;
      ERASRndWipe->Enabled = true;
      ERASPasses->Color = clWindow;
      ERASPasses->Font->Assign(WinFont->Font);
      ERASUpDown->Enabled = ERASRndWipe->Checked;
	  ERASPasses->Enabled = ERASUpDown->Enabled;
      Label19->Enabled = ERASUpDown->Enabled;
   }
   else
   {
      ERASDodWipe->Enabled = false;
	  ERASGutWipe->Enabled = false;
      ERASRndWipe->Enabled = false;
      ERASPasses->Enabled = false;
      ERASPasses->ParentColor = true;
      ERASPasses->Font->Color = clGrayText;
      ERASUpDown->Enabled = false;
      Label19->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::MIXDirectClick(TObject *Sender)
{
   MIXTlsPreferred->Enabled = MIXDirect->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::ERASRndWipeClick(TObject *Sender)
{
   ERASUpDown->Enabled = ERASRndWipe->Checked;
   ERASPasses->Enabled = ERASUpDown->Enabled;
   Label19->Enabled = ERASUpDown->Enabled;
}
//---------------------------------------------------------------------------


void __fastcall TOptionsDlg::GENQuoteCharsChange(TObject *Sender)
{
   Label21->Caption = "\"" + GENQuoteChars->Text + "\"";        
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::MIXUpdateFreqChange(TObject *Sender)
{
   if (MIXUpdateFreq->ItemIndex == 1) // once a day
   {
	  if (MIXUpdate->Checked)
	  {
		 MIXAfterBox->Enabled = true;
		 MIXAfterTime->Enabled = MIXAfterBox->Checked;
	  }
   }
   else
   {
	  MIXAfterBox->Checked = false;
	  MIXAfterBox->Enabled = false;
	  MIXAfterTime->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::MIXAfterBoxClick(TObject *Sender)
{
   MIXAfterTime->Enabled = MIXUpdate->Checked && MIXAfterBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TOptionsDlg::PNGUpdateClick(TObject *Sender)
{
   if (PNGUpdate->Checked)
   {
	  PNGUpdateFreq->Enabled = true;
	  MIXMixMx->Enabled = true;
   }
   else
   {
	  PNGUpdateFreq->Enabled = false;
	  MIXMixMx->Enabled = false;
   }
}
//---------------------------------------------------------------------------
		 
