/*
*  qsdbs.cpp 
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <math.h>
#include <stdio.h>
//#include "overbyteicsmd5.hpp"

#include "qsdbs.h"
#include "gpg.h"
#include "msgmans.h"
#include "utility.h"
#include "dbs.h"
#include "eraser.h"
#include "pgpactdlgs.h"
#include "mixmaster.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TDBSecModeDlg *DBSecModeDlg;
TQSDB *DB;
//---------------------------------------------------------------------------
// flag functions
//---------------------------------------------------------------------------
// BYTE FLAGS
/*void __fastcall SetMessageFlag(AnsiString &key, byte flag)
{
   key[bflags1] |= flag;
}
//---------------------------------------------------------------------------

void __fastcall UnSetMessageFlag(AnsiString &key, byte flag)
{
   key[bflags1] &= (byte)~flag;
}
//---------------------------------------------------------------------------

bool __fastcall MessageFlagIsSet(const AnsiString &key, byte flag)
{
   return (key[bflags1] & flag) == flag;
}*/
//---------------------------------------------------------------------------
// WORD FLAGS
void __fastcall SetMessageFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] |= flag;
}
//---------------------------------------------------------------------------

void __fastcall UnSetMessageFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] &= ~flag;
}
//---------------------------------------------------------------------------

bool __fastcall MessageFlagIsSet(AnsiString key, WORD flag)
{
   return (*(WORD*)&key[wflags] & flag) == flag;
}
//---------------------------------------------------------------------------
// ATTACHMENT FLAGS
void __fastcall SetAttachmentFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] |= flag;
}
//---------------------------------------------------------------------------

void __fastcall UnSetAttachmentFlag(AnsiString &key, WORD flag)
{
   *(WORD*)&key[wflags] &= ~flag;
}
//---------------------------------------------------------------------------

bool __fastcall AttachmentFlagIsSet(AnsiString key, WORD flag)
{
   return (*(WORD*)&key[wflags] & flag) == flag;
}
//---------------------------------------------------------------------------
// WORD FLAGS
void __fastcall SetFlag(WORD &flags, WORD flag)
{
   flags |= flag;
}
//---------------------------------------------------------------------------

void __fastcall UnSetFlag(WORD &flags, WORD flag)
{
   flags &= ~flag;
}
//---------------------------------------------------------------------------

bool __fastcall FlagIsSet(WORD flags, WORD flag)
{
   return (flags & flag) == flag;
}

//---------------------------------------------------------------------------
// QSDB
//---------------------------------------------------------------------------

TQSDB* __fastcall OpenQSDB(const AnsiString &FileName)
{
   assert(FileName != "");
   TQSDB *DB = new TQSDB(FileName);
   if (DB->error)
   {
      delete DB;
	  return NULL;
   }
   return DB; //(TQSDB*)OpenDB(DB);
}
//---------------------------------------------------------------------------

bool __fastcall CloseQSDB(TDB *DB)
{
   return CloseDB(DB);
}

//---------------------------------------------------------------------------
// TQSDB
//---------------------------------------------------------------------------

//__fastcall TQSDB::TQSDB(const AnsiString &FileName, bool InMemory)
//   : TDB(FileName, InMemory)
//{
// unused
//}
//---------------------------------------------------------------------------

__fastcall TQSDB::TQSDB(const AnsiString &FileName)
   : TDB(FileName, true)
{
}
//---------------------------------------------------------------------------

__fastcall TQSDB::~TQSDB()
{
   if (MasterPassword)
   {
      ZeroMemory(MasterPassword->Memory, MasterPassword->Size);
      delete MasterPassword;
   }
}

//---------------------------------------------------------------------------
// Secure Mode Functions
//---------------------------------------------------------------------------

bool __fastcall TQSDB::OpenDatabase()
{
   assert(stream == NULL);
   MasterPassword = NULL;

   AnsiString key, passphrase;
   try
   {
      bool x = (GPGIsFileEncrypted(filename));
//sm("ok");
	  if (x)
      {
         int attempt = 0;
         while (true)
         {
            attempt++;
            if (!GetUserPassphrase(NULL, passphrase))
            {
               error = DB_FAILEDINITERR;
			   return false;
            }

            fstream = new TMemoryStream(), inmemory = true;
            if (!ShowDecryptingDatabase(this, passphrase))
            {
               if (attempt == 3)
               {
                  error = DB_FAILEDINITERR;
                  delete fstream;
                  return false;
               }
               continue;
            }

            MasterPassword = new TMemoryStream();
            break;
		 }
      }
      else
      {
         fstream = new TMemoryStream(), inmemory = true;
         if (!filename.IsEmpty() && FileExists(filename))
            memstream->LoadFromFile(filename);
      }
   }
   catch(Exception &e)
   {
	  exceptmsg = e.Message;
      error = DB_READEXC;
      ShowDBError(error);
      return false;
   }

   dbman = new TDBmanager(this, filename);
   if (!dbman->error)
   {
      index[0] = new TDBindex(dbman);
      modified = false;
   }
   else
   {
      ShowDBError(dbman->error);
      error = dbman->error;
   }

   CreateSessionKey();
   if (MasterPassword)
   {
      GPGSaveMasterPassword(this, passphrase);
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::CloseDatabase()
{
   if (modified && IsSecureMode())
   {
	  AnsiString passphrase;
	  GPGLoadMasterPassword(this, passphrase);
	  ShowEncryptingDatabase(this, passphrase);
	  modified = false;
   }
   ClearSessionKey();

   return TDB::CloseDatabase();
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::GetUserPassphrase(TForm *Owner, AnsiString &passphrase)
{
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   DBSecModeDlg->SetMode(dbpDecrypt);
   DBSecModeDlg->ShowModal();
   bool ok = DBSecModeDlg->ModalResult == mrOk;
   if (ok)
   {
      passphrase = DBSecModeDlg->Phrase->Text;
      DBSecModeDlg->ClearText();
   }
   delete DBSecModeDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::IsSecureMode()
{
   return MasterPassword != NULL;
}
//---------------------------------------------------------------------------

int __fastcall TQSDB::PermissionSecureMode(TForm *Owner)
{
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   AnsiString key;
   LoadSessionKey(key);
   GPGDecryptStreamToString(key, MasterPassword, DBSecModeDlg->currentpassphrase);
   ZeroMemory(key.c_str(), key.Length());
   DBSecModeDlg->SetMode(dbpPermission);
   DBSecModeDlg->ShowModal();
   int ret = DBSecModeDlg->ModalResult;
   delete DBSecModeDlg;
   return ret;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ManageSecureMode(TForm *Owner)
{
   bool secure = IsSecureMode();

   if (secure)
   {
      switch (PermissionSecureMode(Owner))
      {
		 case IDABORT:
			try
			{
			  ZeroMemory(MasterPassword->Memory, MasterPassword->Size);
			}
			__finally
			{
			   delete MasterPassword;
			   MasterPassword = NULL;
			   modified = true;
			   Snooze(50);
			   MsgBox(Owner, "Secure Mode is canceled!   ", MT_INFO, MB_OK);
			   return true;
			}
		 case IDCANCEL:
            return false;
      }
      Snooze(100);
   }

   bool ok = false;
   DBSecModeDlg = new TDBSecModeDlg(Owner);
   DBSecModeDlg->SetMode(secure ? dbpChange : dbpEncrypt);
   DBSecModeDlg->ShowModal();
   if (DBSecModeDlg->ModalResult == mrOk)
   {
      AnsiString key;
      LoadSessionKey(key);
      MasterPassword = new TMemoryStream();
      GPGEncryptStringToStream(key, DBSecModeDlg->currentpassphrase, MasterPassword);
      modified = true;
	  Snooze(100);
      MsgBox(Owner, "Secure Mode is set!    ", MT_INFO, MB_OK);
	  ok = true;
   }
   delete DBSecModeDlg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::CreateSessionKey()
{
   ClearSessionKey();

   AnsiString seskey;
   char pcData[33] = { 0 };

   // get random bytes from mixmaster
   if (rnd_bytes((byte*)pcData, sizeof(pcData) - 1) == 0)
   {
	  seskey = Base64EncodeData(pcData, sizeof(pcData) - 1);
	  ZeroMemory(pcData, sizeof(pcData));
	  assert(seskey.Length() > 32);
   }
   else
   {
	  MsgBox(NULL,
		 "I failed to get random bytes from mixmaster for my session key.  \n"
		 "Using backup method.",
		  MT_INFO, MB_OK);

	  // guaranteed to work but lesser quality
	  // randomize(); called at app startup
	  seskey = AnsiString(time(NULL) * GetTickCount() * _threadid * random(2147483647));
	  seskey = Base64EncodeData(seskey.c_str(), seskey.Length());
   }

   bool ok = PutBuffer(TheSession+DBPS+Key, seskey.c_str(), seskey.Length());
   ZeroMemory(seskey.c_str(), seskey.Length());
   modified = false;

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadSessionKey(AnsiString &SesKey)
{
   TDBbuffer *buf = new TDBbuffer();
   bool ok = GetBuffer(TheSession+DBPS+Key, buf);
   SesKey = buf->data;
   delete buf;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ClearSessionKey()
{
   bool mod = modified;
   bool ok = PutBuffer(TheSession+DBPS+Key, NULL);
   modified = mod;
   return ok;
}
//---------------------------------------------------------------------------

TMemoryStream* __fastcall TQSDB::GetMemoryStream()
{
   TMemoryStream *memstrm = dynamic_cast<TMemoryStream*>(fstream);
   assert(memstrm != NULL);
   return memstrm;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveToFile(const AnsiString &FileName)
{
   if (FileName.IsEmpty() && filename.IsEmpty())
      return true;

   int error = DB_OK;

   if (!FileName.IsEmpty() && FileName != filename)
      filename = FileName;

   try
   {
      // overwrite previous file if present
      bool exists = FileExists(filename);
      TFileStream *filestream = new TFileStream(filename,
         (unsigned short)(exists ? fmOpenReadWrite : fmCreate) | fmShareExclusive);
      filestream->Seek(0, soFromBeginning);
      memstream->SaveToStream(filestream);
      if (filestream->Size > memstream->Size)
         filestream->Size = memstream->Size;
      delete filestream;
      modified = false;
   }
   catch(Exception &e)
   {
      exceptmsg = e.Message;
      error = DB_WRITEEXC;
      return error;
   }

   return !error;
}

//---------------------------------------------------------------------------
// Application Specific Functions
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadConfig(const AnsiString &name, TStrings *config)
{
   TStringList *cfg = new TStringList();
   bool ok = GetText(Config, cfg);
   config->CommaText = cfg->Values[name];
   delete cfg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveConfig(const AnsiString &name, TStrings *config)
{
   TStringList *cfg = new TStringList();
   bool ok = GetText(Config, cfg);
   cfg->Values[name] = config->CommaText;
   if (ok) ok = PutText(Config, cfg);
   delete cfg;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSDB::GetConfigItem(const AnsiString &Section, const AnsiString &Name,
   const AnsiString &DefValue)
{
   TStringList *cfg = new TStringList();
   LoadConfig(Section, cfg);
   AnsiString item = ReadString(cfg, Name, DefValue);
   delete cfg;
   return item;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::GetConfigItem(const AnsiString &Section, const AnsiString &Name, bool DefValue)
{
   TStringList *cfg = new TStringList();
   LoadConfig(Section, cfg);
   bool item = ReadBool(cfg, Name, DefValue);
   delete cfg;
   return item;
}
//---------------------------------------------------------------------------

int __fastcall TQSDB::GetConfigItem(const AnsiString &Section, const AnsiString &Name, int DefValue)
{
   TStringList *cfg = new TStringList();
   LoadConfig(Section, cfg);
   int item = ReadInteger(cfg, Name, DefValue);
   delete cfg;
   return item;
}

//---------------------------------------------------------------------------
// barred remailers
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadBars(const AnsiString &section, TStrings *bars)
{
   return GetText(Bars+DBPS+section, bars);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveBars(const AnsiString &section, TStrings *bars)
{
   return PutText(Bars+DBPS+section, bars);
}

//---------------------------------------------------------------------------
// templates
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadTemplates(TStrings *keys)
{
   return GetText(Templates, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadTemplate(const AnsiString &name, TStrings *msg)
{
   return GetText(Templates+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveTemplate(const AnsiString &name, TStrings *msg)
{
   return PutText(Templates+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::TemplateExists(const AnsiString &name)
{
   return DataExists(Templates+DBPS+name);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelTemplate(const AnsiString &name)
{
   return PutText(Templates+DBPS+name, NULL);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::RenTemplate(const AnsiString &oldname, const AnsiString &newname)
{
   return RenItem(Templates+DBPS+oldname, Templates+DBPS+newname);
}

//---------------------------------------------------------------------------
// notes
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadNotes(TStrings *keys)
{
   return GetText(Notes, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadNote(const AnsiString &name, TStrings *msg)
{
   return GetText(Notes+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveNote(const AnsiString &name, TStrings *msg)
{
   return PutText(Notes+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::NoteExists(const AnsiString &name)
{
   return DataExists(Notes+DBPS+name);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelNote(const AnsiString &name)
{
   return PutText(Notes+DBPS+name, NULL);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::RenNote(const AnsiString &oldname, const AnsiString &newname)
{
   return RenItem(Notes+DBPS+oldname, Notes+DBPS+newname);
}

//---------------------------------------------------------------------------
// drafts
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadDrafts(TStrings *keys)
{
   return GetText(Drafts, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadDraft(const AnsiString &name, TStrings *msg)
{
   return GetText(Drafts+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveDraft(const AnsiString &name, TStrings *msg)
{
   return PutText(Drafts+DBPS+name, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DraftExists(const AnsiString &name)
{
   return DataExists(Drafts+DBPS+name);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelDraft(const AnsiString &name)
{
   return PutText(Drafts+DBPS+name, NULL);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::RenDraft(const AnsiString &oldname, const AnsiString &newname)
{
   return RenItem(Drafts+DBPS+oldname, Drafts+DBPS+newname);
}

//---------------------------------------------------------------------------
// contacts
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadContacts(TStrings *keys)
{
   return GetText(Contacts, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadContact(const AnsiString &email, TStrings *notes)
{
   return GetText(Contacts+DBPS+email, notes);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveContact(const AnsiString &email, TStrings *notes)
{
   return PutText(Contacts+DBPS+email, notes);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelContact(const AnsiString &email)
{
   return DelItem(Contacts+DBPS+email);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ContactExists(const AnsiString &email)
{
   TStringList *keys = new TStringList();
   GetText(Contacts, keys);
   bool exist = keys->IndexOf(email) >= 0;
   delete keys;
   return exist;
}

//---------------------------------------------------------------------------
// Nymservers
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveNymserver(const AnsiString &serverAndType)
{                                          // nym.mixmin.net=ghio
   TStringList *servers = new TStringList();
   bool ok = GetText("nyms"+DBPS+"servers", servers);
   if (ok)
   {
	  AnsiString server, type;
	  server = GetTok(0, serverAndType, '=');
	  type = GetTok(1, serverAndType, '=');
	  servers->Values[server] = type;
	  ok = PutText("nyms"+DBPS+"servers", servers);
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveNymserver(const AnsiString &server, const AnsiString &type)
{
   TStringList *servers = new TStringList();
   bool ok = GetText("nyms"+DBPS+"servers", servers);
   if (ok)
   {
	  servers->Values[server] = type;
	  ok = PutText("nyms"+DBPS+"servers", servers);
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSDB::GetNymserverType(const AnsiString &server)
{
   AnsiString type;
   TStringList *servers = new TStringList();
   if (GetText("nyms"+DBPS+"servers", servers))
	  type = servers->Values[server];
   delete servers;
   return type;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveNymDescriptor(const AnsiString &descrip)
{                                          // nym.mixmin.net=ghio
   TStringList *nyms = new TStringList();
   bool ok = GetText("nyms"+DBPS+"accounts", nyms);
   if (ok)
   {
	  AnsiString address = GetEAddress(descrip);
	  nyms->Values[address] = descrip;
	  ok = PutText("nyms"+DBPS+"accounts", nyms);
   }
   delete nyms;
   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TQSDB::LoadNymDescriptor(const AnsiString &address)
{
   AnsiString descrip;
   TStringList *nyms = new TStringList();
   bool ok = GetText("nyms"+DBPS+"accounts", nyms);
   if (ok)
   {
	  descrip = nyms->Values[address];
   }
   delete nyms;
   return descrip;
}

//---------------------------------------------------------------------------
// default message
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadDefMessage(TStrings *msg)
{
   return GetText(Default+DBPS+Message, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveDefMessage(TStrings *msg)
{
   return PutText(Default+DBPS+Message, msg);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DefMessageExists()
{
   TStringList *msg = new TStringList();
   GetText(Default+DBPS+Message, msg);
   bool ok = msg->Count > 0;
   delete msg;
   return ok;
}

//---------------------------------------------------------------------------
// default article
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadDefArticle(TStrings *art)
{
   return GetText(Default+DBPS+Article, art);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveDefArticle(TStrings *art)
{
   return PutText(Default+DBPS+Article, art);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DefArticleExists()
{
   TStringList *art = new TStringList();
   GetText(Default+DBPS+Article, art);
   bool ok = art->Count > 0;
   delete art;
   return ok;
}

//---------------------------------------------------------------------------
// queues
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadQueueIndex(const AnsiString &Queue, TStrings *keys)
{
   return GetText(Queues+DBPS+Queue, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::UnQueueMessage(const AnsiString &Queue, const AnsiString &Key,
   TStrings *Message)
{
   bool ok = GetText(Queues+DBPS+Queue+DBPS+Key, Message);
   if (ok) ok = PutText(Queues+DBPS+Queue+DBPS+Key, NULL);
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::MoveQueueMessage(const AnsiString &Queue, const AnsiString &Key,
   const AnsiString &BoxName)
{
   TStringList *msg = new TStringList();
   bool ok = UnQueueMessage(OrigQueue, Key, msg);
   if (ok) ok = DelQueueMessage(Queue, Key);
   if (ok) ok = SaveMessage(BoxName, Key, msg);
   delete msg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelQueueMessage(const AnsiString &Queue, const AnsiString &Key)
{
   return PutText(Queues+DBPS+Queue+DBPS+Key, NULL);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadQueueMessage(const AnsiString &Queue, const AnsiString &Key,
   TStrings *Message)
{
   return GetText(Queues+DBPS+Queue+DBPS+Key, Message);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::QueueMessage(const AnsiString &Queue, const AnsiString &Key,
   TStrings *Message)
{
   return PutText(Queues+DBPS+Queue+DBPS+Key, Message);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::QueueMessage(const AnsiString &Key, TStrings *Message,
   TStrings *Original)
{
   bool ok = QueueMessage(MailQueue, Key, Message);
   if (ok) ok = QueueMessage(OrigQueue, Key, Original);
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::IsQueueMessage(const AnsiString &Queue, const AnsiString &Key)
{
   TStringList *items = new TStringList();
   GetText(Queues+DBPS+Queue, items);
   bool ok = items->IndexOf(Key) >= 0;
   delete items;
   return ok;
}
//---------------------------------------------------------------------------

int __fastcall TQSDB::QueueCount(const AnsiString &Queue)
{
   int c = 0;
   TStringList *keys = new TStringList();
   bool ok = LoadQueueIndex(Queue, keys);
   if (ok)
	  c = keys->Count;
   delete keys;
   return c;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ClearQueue(const AnsiString &Queue)
{
   TStringList *msgs = new TStringList();
   bool ok = LoadQueueIndex(Queue, msgs);
   if (ok)
   {
      for (int i = 0; i < msgs->Count; i++)
         PutText(Queues+DBPS+Queue+DBPS+msgs->Strings[i], NULL);
   }
   return ok;
}

//---------------------------------------------------------------------------
// messages & mailboxes
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadMessage(const AnsiString &BoxName, const AnsiString &Key,
   TStrings *lines)
{
   return GetText(Mailboxes+DBPS+BoxName+DBPS+Key, lines);
}
//---------------------------------------------------------------------------

void __fastcall TQSDB::LoadMessage(const AnsiString &BoxName, const AnsiString &Key,
   TDBbuffer *Buffer)
{
   GetBuffer(Mailboxes+DBPS+BoxName+DBPS+Key, Buffer);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveMessage(const AnsiString &BoxName, TStrings *lines)
{
   TMsgManager *msg = new TMsgManager(lines);
   bool ok = PutText(Mailboxes+DBPS+BoxName+DBPS+msg->key, lines);
   delete msg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveMessage(const AnsiString &BoxName, const AnsiString &MsgKey,
   TStrings *lines)
{
   return PutText(Mailboxes+DBPS+BoxName+DBPS+MsgKey, lines);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::MoveMessage(const AnsiString &OldBox, const AnsiString &Key,
   const AnsiString &NewBox)
{
   bool ok; int offset;
   ok = DB->GetDataOffset(Mailboxes+DBPS+OldBox+DBPS+Key, offset);
   if (ok) ok = DB->PutDataOffset(Mailboxes+DBPS+NewBox+DBPS+Key, offset);
   if (ok) ok = DB->DelDataKey(Mailboxes+DBPS+OldBox+DBPS+Key);
   return ok;
}

//---------------------------------------------------------------------------
// also deletes attachments
bool __fastcall TQSDB::DelMessage(const AnsiString &BoxName, const AnsiString &Key)
{
#ifdef QS2
   if (::MessageFlagIsSet(Key, mflAttachment))
   {
	  TStringList *keys = new TStringList();
	  DB->LoadAttachments(Key, keys);
	  for (int i = 0; i < keys->Count; i++)
		 DB->DelAttachment(keys->Strings[i]);
	  delete keys;
   }
#endif
   return DelItem(Mailboxes+DBPS+BoxName+DBPS+Key);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveIncomingMessage(const AnsiString &BoxName, TStrings *lines)
{
   TMsgManager *msg = new TMsgManager(lines);
   bool ok = PutText(Mailboxes+DBPS+BoxName+DBPS+msg->receivekey, lines);
   delete msg;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::NewMailbox(const AnsiString &Mailbox)
{
   TStringList *data = new TStringList();
   LoadMailboxes(data);
   assert(data->IndexOf(Mailbox) < 0);
   data->Clear();
   bool ok =  PutText(Mailboxes+DBPS+Mailbox, data);
   delete data;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::RenMailbox(const AnsiString &OldName, const AnsiString &NewName)
{
   TStringList *boxes = new TStringList();
   LoadMailboxes(boxes);
   assert(OldName != "");
   assert(NewName != "");
   assert(boxes->IndexOf(OldName) >= 0);
   assert(boxes->IndexOf(NewName) < 0);
   delete boxes;
   return SwapKey(Mailboxes+DBPS+OldName, NewName);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelMailbox(const AnsiString &BoxName)
{
   assert(BoxName != "");
   TStringList *keys = new TStringList();
   GetText(Mailboxes+DBPS+BoxName, keys);
   for (int i = 0; i < keys->Count; i++)
	  DelMessage(BoxName, keys->Strings[i]);
   delete keys;
   return DelItem(Mailboxes+DBPS+BoxName);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadMailboxes(TStrings *mailboxes)
{
   return GetText(Mailboxes, mailboxes);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadMailbox(const AnsiString &BoxName, TStrings *messages)
{
   return GetText(Mailboxes+DBPS+BoxName, messages);
}

#ifdef QS2
//---------------------------------------------------------------------------
// Attachments
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadAttachments(TStrings *keys)
{
   return GetText(Attachments, keys);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadAttachments(const AnsiString &Name, TStrings *keys)
{
   assert(Name.Length() >= 15);
   assert(keys != NULL);
   AnsiString name = Name.SubString(1, 15);
   keys->Clear();
   TStringList *attached = new TStringList();
   GetText(Attachments, attached);
   for (int i = 0; i < attached->Count; i++)
   {
	  AnsiString key = attached->Strings[i];
	  if (key.Pos(name) == 1)
		 keys->Add(key);
   }
   return keys->Count;
}
//---------------------------------------------------------------------------

int __fastcall TQSDB::AttachmentCount(const AnsiString &name)
{
   int count;
   TStringList *attaches = new TStringList();
   LoadAttachments(name, attaches);
   count = attaches->Count;
   delete attaches;
   return count;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveAttachment(const AnsiString &name, TMemoryStream *attached)
{
   assert(name != "");
   assert(attached != NULL);
   assert(attached->Size > 0);
   TDBbuffer *buf = new TDBbuffer();
   buf->SetData((char*)attached->Memory, attached->Size);
   bool ok = PutBuffer(Attachments+DBPS+name, buf);
   if (ok)
   {
	  attached->Position = 0;
	  delete buf;
   }
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadAttachment(const AnsiString &datekey, const AnsiString &partno,
   TMemoryStream *attached)
{
   assert(partno != "");
   assert(datekey.Length() == 15);
   assert(attached != NULL);
   TStringList *index = new TStringList();
   bool ok = GetText(Attachments, index);
   if (ok)
   {
	  for (int i = 0; i < index->Count; i++)
	  {
		 AnsiString key = index->Strings[i];

		 if (key.Pos(datekey) == 1 && GetTok(dbapart, key, DBFSc) == partno)
		 {
			TDBbuffer *buf = new TDBbuffer();
			bool ok = GetBuffer(Attachments+DBPS+key, buf);
			if (ok)
			{
			   attached->Position = 0;
			   attached->SetSize(buf->length);
			   attached->Write(buf->data, buf->length);
			   attached->Position = 0;
			}
			delete buf;
			break;
		 }
	  }
   }
   delete index;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadAttachment(const AnsiString &name, TMemoryStream *attached)
{
   assert(name != "");
   assert(attached != NULL);
   TDBbuffer *buf = new TDBbuffer();
   bool ok = GetBuffer(Attachments+DBPS+name, buf);
   if (ok)
   {
	  attached->Position = 0;
	  attached->SetSize(buf->length);
	  attached->Write(buf->data, buf->length);
	  attached->Position = 0;
	  delete buf;
   }
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::AttachmentExists(const AnsiString &datekey, const AnsiString &partno)
{
   assert(partno != "");
   assert(datekey.Length() == 15);
   TStringList *index = new TStringList();
   bool ok = GetText(Attachments, index);
   if (ok)
   {
	  ok = false;
	  for (int i = 0; i < index->Count; i++)
	  {
		 AnsiString key = index->Strings[i];
		 if (key.Pos(datekey) == 1 && GetTok(dbapart, key, DBFSc) == partno)
		 {
			ok = true;
			break;
		 }
	  }
   }
   delete index;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ExportAttachment(const AnsiString &name, const AnsiString &filename)
{
   assert(name != "");
   assert(filename != "");
   TDBbuffer *buf = new TDBbuffer();
   bool ok = GetBuffer(Attachments+DBPS+name, buf);
   if (ok)
   {
	  try
	  {
		 TFileStream *file = new TFileStream(filename, fmCreate);
		 file->Write(buf->data, buf->length);
		 delete file;
	  }
	  catch (Exception &e)
	  {
		 MsgBox(Screen->ActiveForm,
		   "I wasn't able to export your attachment. The error reported was: \n"+
			e.Message, MT_ERROR, MB_OK);
	  }
   }
   delete buf;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelAttachment(const AnsiString &name)
{
   assert(name != "");
   return DelItem(Attachments+DBPS+name);
}
#endif
//---------------------------------------------------------------------------
// Logs
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadLog(const AnsiString &Log, TStrings *Lines)
{
   assert(Log != "");
   assert(Lines != NULL);
   return GetText(MailLogs+DBPS+Log, Lines);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveLog(const AnsiString &Log, TStrings *Lines)
{
   assert(Log != "");
   assert(Lines != NULL);
   return PutText(MailLogs+DBPS+Log, Lines);
}

//---------------------------------------------------------------------------
// SMTP Hosts
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadSMTP(const AnsiString &Server, TStrings *Config)
{
   assert(Server != "");
   assert(Config != NULL);
   TStringList *servers = new TStringList();
   bool ok = GetText(SMTPHosts, servers);
   if (ok && servers->Count)
   {
	  if (Server.IsEmpty())
		 Config->CommaText = servers->Values[servers->Names[0]];
	  else
		 Config->CommaText = servers->Values[Server];
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadSMTP(TStrings *servers)
{
   assert(servers != NULL);
   return GetText(SMTPHosts, servers);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveSMTP(TStrings *servers)
{
   return PutText(SMTPHosts, servers);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SMTPExists(const AnsiString &Server)
{
   assert(Server != "");
   TStringList *servers = new TStringList();
   GetText(SMTPHosts, servers);
   bool ok = servers->Values[Server] != "";
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveSMTP(const AnsiString &Server, TStrings *Config)
{
   assert(Server != "");
   assert(Config != NULL);
   TStringList *servers = new TStringList();
   bool ok = GetText(SMTPHosts, servers);
   if (ok)
   {
	  if (Server.IsEmpty())
	  {
		 AnsiString cfgline = Config->Values["key"] + "=" + Config->CommaText;
		 if (servers->Count)
			servers->Strings[0] = cfgline;
		 else
			servers->Add(cfgline);
	  }
	  else
	  {
		 servers->Values[Server] = Config->CommaText;
	  }
	  ok = PutText(SMTPHosts, servers);
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelSMTP(const AnsiString &Server)
{
   TStringList *servers = new TStringList();
   bool ok = GetText(SMTPHosts, servers);
   if (ok)
   {
	  servers->Values[Server] = "";
	  ok = PutText(SMTPHosts, servers);
   }
   delete servers;
   return ok;
}

//---------------------------------------------------------------------------
// POP3 Hosts
//---------------------------------------------------------------------------
#ifdef QS2
bool __fastcall TQSDB::LoadPOP3(const AnsiString &Server, TStrings *Config)
{
   TStringList *servers = new TStringList();
   bool ok = GetText(POP3Hosts, servers);
   if (ok && servers->Count)
   {
	  if (Server.IsEmpty())
		 Config->CommaText = servers->Values[servers->Names[0]];
	  else
		 Config->CommaText = servers->Values[Server];
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadPOP3(TStrings *servers)
{
   return GetText(POP3Hosts, servers);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SavePOP3(TStrings *servers)
{
   return PutText(POP3Hosts, servers);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::POP3Exists(const AnsiString &Server)
{
   TStringList *servers = new TStringList();
   GetText(POP3Hosts, servers);
   bool ok = servers->Values[Server] != "";
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SavePOP3(const AnsiString &Server, TStrings *Config)
{
   TStringList *servers = new TStringList();
   bool ok = GetText(POP3Hosts, servers);
   if (ok)
   {
	  if (Server.IsEmpty())
	  {
		 AnsiString cfgline = Config->Values["key"] + "=" + Config->CommaText;
		 if (servers->Count)
			servers->Strings[0] = cfgline;
		 else
			servers->Add(cfgline);
	  }
	  else
	  {
		 servers->Values[Server] = Config->CommaText;
	  }
	  ok = PutText(POP3Hosts, servers);
   }
   delete servers;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelPOP3(const AnsiString &Server)
{
   TStringList *servers = new TStringList();
   bool ok = GetText(POP3Hosts, servers);
   if (ok)
   {
	  servers->Values[Server] = "";
	  ok = PutText(POP3Hosts, servers);
   }
   delete servers;
   return ok;
}
#endif
//---------------------------------------------------------------------------
// proxies
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadProxies(TStrings *proxies)
{
   return GetText(Proxies, proxies);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveProxies(TStrings *proxies)
{
   return PutText(Proxies, proxies);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadProxy(const AnsiString &name, TStrings *proxy)
{
   TStringList *proxies = new TStringList();
   bool ok = GetText(Proxies, proxies);
   proxy->CommaText = proxies->Values[name];
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveProxy(const AnsiString &name, TStrings *proxy)
{
   TStringList *proxies = new TStringList();
   bool ok = GetText(Proxies, proxies);
   proxies->Values[name] = proxy->CommaText;
   if (ok) ok = PutText(Proxies, proxies);
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::ProxyExists(const AnsiString &Server)
{
   TStringList *proxies = new TStringList();
   GetText(Proxies, proxies);
   bool ok = proxies->Values[Server] != "";
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::DelProxy(const AnsiString &Server)
{
   TStringList *proxies = new TStringList();
   bool ok = GetText(Proxies, proxies);
   if (ok)
   {
      proxies->Values[Server] = "";
      ok = PutText(Proxies, proxies);
   }
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::GetProxyNames(TStrings *HTTP, TStrings *SMTP, TStrings *POP3, TStrings *NNTP)
{
   assert(HTTP || SMTP || POP3 || NNTP);
   if (HTTP)
	   HTTP->Clear();
   if (SMTP)
	   SMTP->Clear();
   if (POP3)
	   POP3->Clear();
   if (NNTP)
	   NNTP->Clear();

   TStringList *proxies = new TStringList();
   bool ok = LoadProxies(proxies);
   if (ok)
   {
	  TStringList *proxy = new TStringList();
	  for (int i = 0; i < proxies->Count; i++)
	  {
		 AnsiString name = proxies->Names[i];
		 proxy->CommaText = proxies->Values[name];
		 if (SMTP && proxy->Values["smtpproxy"] == "1")
			SMTP->Add(name);
		 if (HTTP && proxy->Values["httpproxy"] == "1")
			HTTP->Add(name);
		 if (POP3 && proxy->Values["pop3proxy"] == "1")
			POP3->Add(name);
		 if (NNTP && proxy->Values["nntpproxy"] == "1")
			NNTP->Add(name);
	  }
	  delete proxy;
   }
   delete proxies;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TQSDB::FlagMessage(const AnsiString &Mailbox, const AnsiString &Key,
   byte flag)
{
   FlagKey(Mailboxes+DBPS+Mailbox+DBPS+Key, bflags1, flag);
}
//---------------------------------------------------------------------------

void __fastcall TQSDB::UnFlagMessage(const AnsiString &Mailbox, const AnsiString &Key,
   byte flag)
{
   UnFlagKey(Mailboxes+DBPS+Mailbox+DBPS+Key, bflags1, flag);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::LoadNewsProxy(TStrings *Proxy)
{
   return GetText(NEWSProxy, Proxy);
}
//---------------------------------------------------------------------------

bool __fastcall TQSDB::SaveNewsProxy(TStrings *Proxy)
{
   return PutText(NEWSProxy, Proxy);
}

//---------------------------------------------------------------------------
// TDBPasswordDlg2
//---------------------------------------------------------------------------

__fastcall TDBSecModeDlg::TDBSecModeDlg(TComponent* Owner)
		: TForm(Owner)
{
   Position = Owner == NULL ? poScreenCenter : poOwnerFormCenter;
   if (Owner)
      Font->Assign(((TForm*)Owner)->Font);

   minpasslen = GetAdvancedConfigItem(GPGminpasslength, GPGMinPassLength);
}
//---------------------------------------------------------------------------

__fastcall TDBSecModeDlg::~TDBSecModeDlg()
{
   ClearText();
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::SetMode(int amode)
{
   mode = amode;
   switch (mode)
   {
      case dbpDecrypt:
         Caption = " QuickSilver Secure Mode ";
         Label2->Caption = "&Enter decryption passphrase :";
         QualityBox->Hide();
         Cnfirm->Hide();
         Label3->Hide();
         //HelpBtn->Hide();
         Label2->Top = QualityBox->Top;
         HideTypeBox->Top = Label2->Top;
         Phrase->Top = Label2->Top + 18;
         OkBtn->Top = Phrase->Top + Phrase->Height + 15;
         CancelBtn->Top = OkBtn->Top;
         CancelBtn->Left = 260;
         OkBtn->Left = 178;
         Height = 178;
         Width = 355;
         break;
      case dbpPermission:
         Caption = " Secure Mode Permission ";
         Label2->Caption =
            "In order to quit Secure Mode or change the passphrase you need to "
            "enter the current passphrase now :";
         QualityBox->Hide();
         Cnfirm->Hide();
         Label3->Hide();
         //HelpBtn->Hide();
         Label2->Top = QualityBox->Top;
         Phrase->Top = Label2->Top + 65;
         HideTypeBox->Top = Phrase->Top - 20;
         OkBtn->Top = Phrase->Top + Phrase->Height + 12;
         CancelBtn->Top = OkBtn->Top;
         CancelBtn->Left = 260;
         OkBtn->Left = 178;
         ClearBox->Top = OkBtn->Top;
         ClearBox->Show();
         Height = 235;
         Width = 355;
         break;
      case dbpChange:
         Caption = " Secure Mode Change Passphrase ";
         Label2->Caption = "&Enter new passphrase :";
         break;
      case dbpEncrypt:
         Caption = " Secure Mode ";
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::ClearText()
{
   ProgressBar1->Position = 0;
   ZeroMemory(currentpassphrase.c_str(), currentpassphrase.Length());
   currentpassphrase = "";
   Phrase->Clear();
   Cnfirm->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      TIniFile *ini;
      switch (mode)
      {
         case dbpEncrypt:
         case dbpChange:
            if (Phrase->Text.IsEmpty())
            {
               MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
               ActiveControl = Phrase;
               CanClose = false;
               return;
            }
            if (Phrase->Text.Length() < minpasslen)
            {
               MsgBox(this,
                 "Your passphrase needs to be at least " + IntToStr(minpasslen) +
                 " characters long!   ",
                  MT_INFO, MB_OK);
               ActiveControl = Phrase;
               CanClose = false;
               ClearText();
               return;
            }
            if (Cnfirm->Text != Phrase->Text)
            {
               MsgBox(this, "Your passphrase entries don't match.   ",
                  MT_INFO, MB_OK);
               ActiveControl = Phrase;
               CanClose = false;
               ClearText();
               return;
            }
            break;
         case dbpDecrypt:
            if (Phrase->Text.IsEmpty())
            {
               MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
               ActiveControl = Phrase;
               CanClose = false;
               ClearText();
               return;
            }
            break;
         case dbpPermission:
            if (Phrase->Text.IsEmpty())
            {
               MsgBox(this, "Enter a passphrase or click Cancel!   ", MT_INFO, MB_OK);
               ActiveControl = Phrase;
               CanClose = false;
               return;
            }
            if (Phrase->Text != currentpassphrase)
            {
               MsgBox(this, "Bad passphrase!        ", MT_INFO, MB_OK);
               attempts++;
               if (attempts < 3)
               {
                  ActiveControl = Phrase;
                  CanClose = false;
                  Phrase->Clear();
                  return;
               }
               ModalResult = mrCancel;
               return;
            }
            if (ClearBox->Checked)
            {
               switch (MsgBox(this,
                  "IMPORTANT!\n"
                  "I'm going to CANCEL Secure Mode. Your database will no longer   \n"
                  "be encrypted between sessions. Is that what you want?   ",
                  MT_CONFIRM, MB_YESNO))
               {
                  case IDNO:
                     CanClose = false;
                     ActiveControl = Phrase;
                     Phrase->Clear();
                     break;
                  case IDYES:
                     ModalResult = mrAbort;
                     break;
               }
               return;
            }
            break;
      }
      currentpassphrase = Phrase->Text;
   }
}
//---------------------------------------------------------------------------
// This function is not written by me (Richard). I swiped it from PGP source code
// That is something I've only done twice. The other is the Isacc Generator in
// wipe.cpp. It was public domain.

#define NUMRANGES	8
UINT PGPcomdlgPassphraseQuality(LPSTR szPhrase)
{
	UINT quality;
	unsigned short charIndex, rangeIndex, span;
	typedef struct ScoreStruct
	{
		unsigned char firstChar;
		unsigned char lastChar;
	} Range;
	static Range sRanges[] =
	{
		{ 0,		' ' - 1},
		{ ' ',		'A' - 1},
		{ 'A',		'Z'},
		{ 'Z' + 1,	'a' - 1},
		{ 'a',		'z'},
		{ '0',		'9'},
		{ 'z' + 1,	127},
		{ 128,		255},
	};
	BOOL haveRange[NUMRANGES];

        memset(haveRange,'\0',sizeof(haveRange));
        //pgpClearMemory (haveRange, sizeof (haveRange));
	for (charIndex = 1; charIndex < lstrlen (szPhrase); ++charIndex) {
		for (rangeIndex = 0; rangeIndex < NUMRANGES; ++rangeIndex)	{
			UCHAR			theChar;
			Range *			theRange;

			theChar		= szPhrase[charIndex];
			theRange	= &sRanges[rangeIndex];

			if (theChar >= theRange->firstChar &&
				theChar <= theRange->lastChar)
			{
				haveRange[rangeIndex] = TRUE;
				break;
			}
		}
	}

	// determine the total span
	span = 0;
	for (rangeIndex = 0; rangeIndex < NUMRANGES; ++rangeIndex) {
		if (haveRange[rangeIndex]) {
			span += (unsigned short)(1+(sRanges[rangeIndex].lastChar-
				sRanges[rangeIndex].firstChar));
		}
	}

	if (span != 0) {
		double	bitsPerChar	= log(span) / log (2.0);  // base 2 log of span
		ULONG	totalBits	= (ULONG)(bitsPerChar * lstrlen (szPhrase));

		// assume a maximum quality of 128 bits
		quality = (totalBits * 100) / 128;
		if (quality > 100) quality = 100;
	}
	else {
		quality = 0;
	}

	return (quality);
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::PhraseChange(TObject *Sender)
{
   ProgressBar1->Position = (WORD)PGPcomdlgPassphraseQuality(Phrase->Text.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::HideTypeBoxClick(TObject *Sender)
{
   Phrase->Font->Color = HideTypeBox->Checked ? clWindow : clBlack;
   Cnfirm->Font->Color = Phrase->Font->Color;
}
//---------------------------------------------------------------------------

void __fastcall TDBSecModeDlg::FormShow(TObject *Sender)
{
   HideTypeBoxClick(HideTypeBox);
}

//---------------------------------------------------------------------------
// For Session Keys
//---------------------------------------------------------------------------

AnsiString __fastcall Base64EncodeData(char *buffer, int length)
{
   const char Base64Chars[] =
   {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   char DataIn[3];
   char DataOut[81];
   AnsiString encodedline;
   int i, Count = 0, BytesRd, InBytes = sizeof(DataIn);

   TMemoryStream *f = new TMemoryStream();
   f->Write(buffer, length);
   f->Position = 0;

   while (true)
   {
      BytesRd = f->Read(DataIn, InBytes);
      for (i = 1; i < 4; i++)
         DataOut[Count + i] = 0x40;

      DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
      DataOut[Count + 1] = char((DataIn[0] & 0x03) << 4);
      if (BytesRd > 1)
      {
         DataOut[Count + 1] += char((DataIn[1] & 0xF0) >> 4);
         DataOut[Count + 2]  = char((DataIn[1] & 0x0F) << 2);
         if (BytesRd > 2)
         {
            DataOut[Count + 2] += char((DataIn[2] & 0xC0) >> 6);
            DataOut[Count + 3]  =  char(DataIn[2] & 0x3F);
         }
      }

      for (i = 0; i < 4; i++)
         DataOut[Count + i] = byte(Base64Chars[DataOut[Count + i]]);

      Count += 4;
      if (Count < 60 && BytesRd == InBytes)
         continue;

      DataOut[Count] = '\0';

      encodedline += AnsiString(&DataOut[0]);

      if (BytesRd < InBytes)
         break;

      Count = 0;
   }

   delete f;
   return encodedline;
}
//---------------------------------------------------------------------------

