/*
*  setupdlgs2.cpp
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <shlobj.h>

#include "setupdlgs2.h"
#include "qslver.h"
#include "global.h"
#include "utility.h"
#include "msgmans.h"
#include "qsdbs.h"
#include "httpgetdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSetupDlg2 *SetupDlg2;
//---------------------------------------------------------------------------

bool __fastcall RunQuickSilverSetup(TForm *Owner)
{
   TSetupDlg2 *setup = new TSetupDlg2(Owner);
   setup->ShowModal();
   bool ok = setup->ModalResult == mrOk;
   delete setup;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TSetupDlg2::TSetupDlg2(TComponent* Owner)
        : TForm(Owner)
{
   Caption = QuickSilverVersion();
   if (Owner)
   {
      Position = poOwnerFormCenter;
      PopupParent = (TForm*)Owner;
   }
   PageControl->ActivePage = Start;
   ActiveControl = EmailAddress;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::NextBtnClick(TObject *Sender)
{
   TTabSheet *page = PageControl->ActivePage;
   if (page == Finish)
   {
      SaveSetup();
	  ModalResult = mrOk;
	  return;
   }

   if (page == Start && StartPageValid())
   {
	  PageControl->ActivePage = SMTPHost;
	  ActiveControl = SMTPServer;
   }
   else if (page == SMTPHost && SMTPHostPageValid())
   {
	  PageControl->ActivePage = SMTPProxy;
	  ActiveControl = SMTPProxyBox;
   }
   else if (page == SMTPProxy && SMTPProxyPageValid())
   {
	  PageControl->ActivePage = HTTPProxy;
	  ActiveControl = HTTPProxyBox;
   }
   else if (page == HTTPProxy && HTTPProxyPageValid())
   {
	  GetQSDefaultsPage();
	  LoadFinishSpec();
	  PageControl->ActivePage = Finish;
   }
   NextBtn->Caption = PageControl->ActivePage != Finish ? "Next" : "Finish";
   BackBtn->Enabled = PageControl->ActivePage != Start;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::GetQSDefaultsPage()
{
   /*MsgBox(this,
	 "I'm going online to fetch a page of default values for QS setup.\n"
	 "It will only take a second. If you defined an HTTP proxy, I'll use\n"
	 "it for the transfer.",
	  MT_INFO, MB_OK
   );
   TStringList *defs = new TStringList();
   while (true)
   {
	  if (GetDefaultsTxtPage(this, QSDefaultsURL, defs))
	  {
		 defs->SaveToFile(AppDataDir()+QSDefaultsFILE);
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteBool("settings", "gotdefaults", true);
		 delete ini;
		 break;
	  }
	  else
	  {
		 MsgBox(this,
			"I wasn't able to get the defaults file I need! but don't\n"
			"worry about it. I have values I can use anyway, and I'll try\n"
			"again next time you start QS.",
			 MT_INFO, MB_OK
		 );
		 break;
	  }
   }
   delete defs;*/
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::BackBtnClick(TObject *Sender)
{
   TTabSheet *page = PageControl->ActivePage;
   if (page == Finish)
   {
	  PageControl->ActivePage = HTTPProxy;
	  ActiveControl = HTTPProxyBox;
   }
   else if (page == HTTPProxy)
   {
      PageControl->ActivePage = SMTPProxy;
      ActiveControl = SMTPProxyBox;
   }
   else if (page == SMTPProxy)
   {
      PageControl->ActivePage = SMTPHost;
      ActiveControl = SMTPServer;
   }
   else if (page == SMTPHost)
   {
      PageControl->ActivePage = Start;
      ActiveControl = EmailAddress;
   }
   NextBtn->Caption = "Next";
   BackBtn->Enabled = PageControl->ActivePage != Start;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::LoadFinishSpec()
{
   Spec->Clear();
   Spec->Lines->Add(" ");
   Spec->Lines->Add(" Email Address = " + EmailAddress->Text);
   Spec->Lines->Add(" ");
   Spec->Lines->Add(" SMTP Server = " + SMTPServer->Text);
   Spec->Lines->Add(" SMTP Port = " + SMTPPort->Text);
   Spec->Lines->Add(" SMTP TLS/SSL = " + SMTPTls->Text);
   if (SMTPAuth->Checked)
   {
	  Spec->Lines->Add(" SMTP User ID = " + SMTPUserID->Text);
	  Spec->Lines->Add(" SMTP Password = " +
		 AnsiString().StringOfChar('*', SMTPPassword->Text.Length()));
   }
   Spec->Lines->Add(" ");
   bool proxy = SMTPProxyBox->Checked;
   Spec->Lines->Add(" SMTP Proxy = " + AnsiString(proxy ? "yes" : "none"));
   if (proxy)
   {
	  Spec->Lines->Add(" SMTP Proxy Server = " + SProxyServer->Text);
	  Spec->Lines->Add(" SMTP Proxy Port = " + SProxyPort->Text);
	  Spec->Lines->Add(" SMTP Proxy Socks Level = " + SProxySocks->Text);
	  Spec->Lines->Add(" SMTP Proxy TOR = " +
		 AnsiString(SProxyTOR->Checked ? "yes" : "no"));
	  if (SProxyAuth->Checked)
	  {
		Spec->Lines->Add(" SMTP Proxy User ID = " + SProxyUserID->Text);
		Spec->Lines->Add(" SMTP Proxy Password = " +
		   AnsiString().StringOfChar('*', SProxyPassword->Text.Length()));
	  }
   }
   Spec->Lines->Add(" ");
   proxy = HTTPProxyBox->Checked;
   Spec->Lines->Add(" HTTP Proxy = " + AnsiString(proxy ? "yes" : "none"));
   if (proxy)
   {
	  Spec->Lines->Add(" HTTP Proxy Server = " + HProxyServer->Text);
	  Spec->Lines->Add(" HTTP Proxy Port = " + HProxyPort->Text);
	  Spec->Lines->Add(" HTTP Proxy Socks Level = " + HProxySocks->Text);
      Spec->Lines->Add(" HTTP Proxy TOR = " +
         AnsiString(HProxyTOR->Checked ? "yes" : "no"));
      if (SProxyAuth->Checked)
      {
        Spec->Lines->Add(" HTTP Proxy User ID = " + HProxyUserID->Text);
		Spec->Lines->Add(" HTTP Proxy Password = " +
           AnsiString().StringOfChar('*', HProxyPassword->Text.Length()));
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrCancel)
   {
      if (MsgBox(this,
         "You haven't completed QS setup. The program won't be installed \n"
         "completely, but you can finish later by just running QS.\n\n"
         "Are you sure you want to quit now?",
         MT_CONFIRM, MB_YESNO) == IDNO)
      {
         CanClose = false;
         return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SMTPHideTypeClick(TObject *Sender)
{
   SMTPPassword->PasswordChar = SMTPHideType->Checked ? '*' : '\0';
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SProxyHideTypeClick(TObject *Sender)
{
   SProxyPassword->PasswordChar = SProxyHideType->Checked ? '*' : '\0';
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::HProxyHideTypeClick(TObject *Sender)
{
   HProxyPassword->PasswordChar = HProxyHideType->Checked ? '*' : '\0';
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SMTPAuthClick(TObject *Sender)
{
   bool chkd = SMTPAuth->Checked;
   SMTPUserID->Enabled = chkd;
   SMTPPassword->Enabled = chkd;
   SMTPHideType->Enabled = chkd;
   Label5->Enabled = chkd;
   Label2->Enabled = chkd;
}
//---------------------------------------------------------------------------

bool __fastcall TSetupDlg2::StartPageValid()
{
   TMsgManager *msg = new TMsgManager(NULL);
   if (EmailAddress->Text.IsEmpty())
   {
      MsgBox(this,
        "The Email Address field is required!   ",
         MT_INFO, MB_OK);
      ActiveControl = EmailAddress;
      delete msg;
      return false;
   }
   if (!msg->ValidAddress(EmailAddress->Text))
   {
      MsgBox(this,
        "Sorry to say, your Email Address doesn't appear to be valid!   ",
         MT_INFO, MB_OK);
      ActiveControl = EmailAddress;
	  delete msg;
	  return false;
   }
   delete msg;
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSetupDlg2::SMTPHostPageValid()
{
   TMsgManager *msg = new TMsgManager(NULL);
   if (SMTPServer->Text.IsEmpty())
   {
      MsgBox(this,
        "The SMTP Host field is required!   ",
         MT_INFO, MB_OK);
      ActiveControl = SMTPServer;
      delete msg;
      return false;
   }
   if (!msg->ValidHost(SMTPServer->Text))
   {
      MsgBox(this,
        "Sorry to say, your SMTP Server doesn't appear to be valid!   ",
         MT_INFO, MB_OK);
      ActiveControl = SMTPServer;
      delete msg;
      return false;
   }
   delete msg;

   if (SMTPPort->Text.IsEmpty())
   {
      MsgBox(this,
        "The SMTP Port field is required!   ",
         MT_INFO, MB_OK);
      ActiveControl = SMTPPort;
      return false;
   }

   if (SMTPAuth->Checked)
   {
      if (SMTPUserID->Text.IsEmpty())
      {
         MsgBox(this,
           "With Authentication enabled, the User ID field is required!   ",
			MT_INFO, MB_OK);
         ActiveControl = SMTPUserID;
         return false;
      }
      if (SMTPPassword->Text.IsEmpty())
      {
         MsgBox(this,
           "With Authentication enabled, the Password field is required!   ",
            MT_INFO, MB_OK);
         ActiveControl = SMTPPassword;
         return false;
      }
   }

   if (SMTPPort->Text == "587" && SMTPTls->Text != "TLS")
   {
      if (MsgBox(this,
         "You have SMTP port 587 but TLS/SSL is not set to TLS. Normally, port  \n"
         "587 is used for TLS connections. Shall I change it for you?",
          MT_CONFIRM, MB_YESNO) == IDYES)
      {
         SMTPTls->ItemIndex = 1;
      }
   }
   if (SMTPPort->Text == "465" && SMTPTls->Text != "SSL")
   {
      if (MsgBox(this,
         "You have SMTP port 465 but TLS/SSL is not set to SSL. Normally, port  \n"
         "465 is used for SSL connections. Shall I change it for you?",
          MT_CONFIRM, MB_YESNO) == IDYES)
      {
         SMTPTls->ItemIndex = 2;
	  }
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSetupDlg2::SMTPProxyPageValid()
{
   if (SMTPProxyBox->Checked)
   {
      if (SProxyServer->Text.IsEmpty())
      {
         MsgBox(this,
           "With SMTP Proxy enabled, the Proxy Server field is required!   ",
            MT_INFO, MB_OK);
         ActiveControl = SProxyServer;
         return false;
      }

      TMsgManager *msg = new TMsgManager(NULL);
      if (!msg->ValidHost(SProxyServer->Text))
      {
         MsgBox(this,
           "Sorry to say, your Proxy Server doesn't appear valid!  ",
            MT_INFO, MB_OK);
         ActiveControl = SProxyServer;
         delete msg;
         return false;
      }
      delete msg;

      if (SProxyPort->Text.IsEmpty())
      {
         MsgBox(this,
           "With SMTP Proxy enabled, the Port field is required!   ",
            MT_INFO, MB_OK);
         ActiveControl = SProxyPort;
         return false;
	  }

      if (SProxyAuth->Checked)
      {
         if (SProxyUserID->Text.IsEmpty())
         {
            MsgBox(this,
			  "With Authentication enabled User ID is a required field!   ",
               MT_INFO, MB_OK);
            ActiveControl = SProxyUserID;
            return false;
         }
         if (SProxyPassword->Text.IsEmpty())
         {
            MsgBox(this,
              "With Authentication enabled Password is a required field!   ",
               MT_INFO, MB_OK);
            ActiveControl = SProxyPassword;
            return false;
         }
      }

      if (SProxyServer->Text == "127.0.0.1" && SProxyPort->Text == "9050" &&
         !SProxyTOR->Checked)
      {
         if (MsgBox(this,
           "Your Proxy Host is 127.0.0.1 and your Port is 9050.   \n"
           "Is this a TOR proxy?",
            MT_CONFIRM, MB_YESNO) == IDYES)
         {
            SProxyTOR->Checked = true;
            SProxySocks->ItemIndex = 1;
         }
      }

      if (SProxyTOR->Checked && SProxySocks->ItemIndex != 1)
      {
         if (MsgBox(this,
           "You have the TOR proxy box ticked but you don't have SocksLevel 4a  \n"
		   "selected. With TOR it should be 4a. Shall I change it?",
			MT_CONFIRM, MB_YESNO) == IDYES)
         {
            SProxySocks->ItemIndex = 1;
         }
      }
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSetupDlg2::HTTPProxyPageValid()
{
   if (HTTPProxyBox->Checked)
   {
      if (HProxyServer->Text.IsEmpty())
      {
         MsgBox(this,
           "With HTTP Proxy enabled, the Proxy Server field is required!   ",
            MT_INFO, MB_OK);
         ActiveControl = HProxyServer;
         return false;
      }

      TMsgManager *msg = new TMsgManager(NULL);
      if (!msg->ValidHost(HProxyServer->Text))
      {
         MsgBox(this,
           "Sorry to say, your Proxy Server doesn't appear valid!  ",
            MT_INFO, MB_OK);
         ActiveControl = HProxyServer;
         delete msg;
         return false;
      }
      delete msg;

      if (HProxyPort->Text.IsEmpty())
      {
         MsgBox(this,
		   "With HTTP Proxy enabled, the Port field is required!   ",
			MT_INFO, MB_OK);
         ActiveControl = HProxyPort;
         return false;
      }

      if (HProxyAuth->Checked)
	  {
         if (HProxyUserID->Text.IsEmpty())
         {
            MsgBox(this,
              "With Authentication enabled User ID is a required field!   ",
               MT_INFO, MB_OK);
            ActiveControl = HProxyUserID;
            return false;
         }
         if (HProxyPassword->Text.IsEmpty())
         {
            MsgBox(this,
              "With Authentication enabled Password is a required field!   ",
               MT_INFO, MB_OK);
            ActiveControl = HProxyPassword;
            return false;
         }
      }

      if (HProxyServer->Text == "127.0.0.1" && HProxyPort->Text == "9050" &&
         !HProxyTOR->Checked)
      {
         if (MsgBox(this,
           "Your Proxy Host is 127.0.0.1 and your Port is 9050.   \n"
           "Is this a TOR proxy?",
            MT_CONFIRM, MB_YESNO) == IDYES)
         {
            HProxyTOR->Checked = true;
            HProxySocks->ItemIndex = 2;
         }
      }

	  if (HProxyTOR->Checked && HProxySocks->ItemIndex != 2)
	  {
         if (MsgBox(this,
           "You have the TOR proxy box ticked but you don't have SocksLevel 4a  \n"
           "selected. With TOR it should be 4a. Shall I change it?",
            MT_CONFIRM, MB_YESNO) == IDYES)
         {
			HProxySocks->ItemIndex = 1;
         }
      }
   }
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SMTPPortChange(TObject *Sender)
{
   if (SMTPPort->Text == "587")
      SMTPTls->ItemIndex = 1;
   else if (SMTPPort->Text == "465")
      SMTPTls->ItemIndex = 2;
   else
      SMTPTls->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SMTPProxyBoxClick(TObject *Sender)
{
   bool chkd = SMTPProxyBox->Checked;
   SProxyServer->Enabled = chkd;
   SProxyPort->Enabled = chkd;
   SProxySocks->Enabled = chkd;
   SProxyTOR->Enabled = chkd;
   Label9->Enabled = chkd;
   Label10->Enabled = chkd;
   Label11->Enabled = chkd;
   SProxyAuth->Enabled = chkd;
   bool achkd = SProxyAuth->Checked;
   SProxyUserID->Enabled = chkd && achkd;
   SProxyPassword->Enabled = chkd && achkd;
   SProxyHideType->Enabled = chkd && achkd;
   Label8->Enabled = chkd && achkd;
   Label7->Enabled = chkd && achkd;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SProxyAuthClick(TObject *Sender)
{
   bool chkd = SProxyAuth->Checked;
   SProxyUserID->Enabled = chkd;
   SProxyPassword->Enabled = chkd;
   SProxyHideType->Enabled = chkd;
   Label8->Enabled = chkd;
   Label7->Enabled = chkd;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::HTTPProxyBoxClick(TObject *Sender)
{
   bool chkd = HTTPProxyBox->Checked;
   HProxySameBtn->Enabled = chkd && SMTPProxyBox->Checked;
   HProxyServer->Enabled = chkd;
   HProxyPort->Enabled = chkd;
   HProxySocks->Enabled = chkd;
   HProxyTOR->Enabled = chkd;
   Label13->Enabled = chkd;
   Label14->Enabled = chkd;
   Label16->Enabled = chkd;
   HProxyAuth->Enabled = chkd;
   bool achkd = HProxyAuth->Checked;
   HProxyUserID->Enabled = chkd && achkd;
   HProxyPassword->Enabled = chkd && achkd;
   HProxyHideType->Enabled = chkd && achkd;
   Label15->Enabled = chkd && achkd;
   Label12->Enabled = chkd && achkd;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::HProxyAuthClick(TObject *Sender)
{
   bool chkd = HProxyAuth->Checked;
   HProxyUserID->Enabled = chkd;
   HProxyPassword->Enabled = chkd;
   HProxyHideType->Enabled = chkd;
   Label15->Enabled = chkd;
   Label12->Enabled = chkd;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::HProxySameBtnClick(TObject *Sender)
{
   assert(SMTPProxyBox->Checked);
   HProxyServer->Text = SProxyServer->Text;
   HProxyPort->Text = SProxyPort->Text;
   HProxySocks->ItemIndex = SProxySocks->ItemIndex + 1;
   HProxyTOR->Checked = SProxyTOR->Checked;
   HProxyAuth->Checked = SProxyAuth->Checked;
   HProxyUserID->Text = SProxyUserID->Text;
   HProxyPassword->Text = SProxyPassword->Text;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SProxyTORClick(TObject *Sender)
{
   SProxySocks->ItemIndex = 1;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::HProxyTORClick(TObject *Sender)
{
   HProxySocks->ItemIndex = 2;
}
//---------------------------------------------------------------------------

bool __fastcall TSetupDlg2::ProxiesSame()
{
   bool ok =
	  SMTPProxyBox->Checked && HTTPProxyBox->Checked &&
	  SProxyServer->Text == HProxyServer->Text &&
      SProxySocks->Text == HProxySocks->Text &&
      SProxyTOR->Checked == HProxyTOR->Checked &&
      SProxyAuth->Checked == HProxyAuth->Checked;
   if (ok && SProxyAuth->Checked)
   {
	  ok = SProxyUserID->Text == HProxyUserID->Text &&
         SProxyPassword->Text == SProxyPassword->Text;
   }
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::CreateDesktopShortcut()
{
   PItemIDList PIDL;
   char InFolder[MAX_PATH];
   WideString LinkName;

   // get the path to users desktop
   SHGetSpecialFolderLocation(0, CSIDL_DESKTOPDIRECTORY, &PIDL) ;
   SHGetPathFromIDList(PIDL, InFolder) ;
   LinkName = InFolder + AnsiString("\\QuickSilver Lite.lnk");

   HRESULT hres;
   IShellLink* psl;
   // Get a pointer to the IShellLink interface.
   hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                           IID_IShellLink, (void**)(&psl));
   if (SUCCEEDED(hres))
   {
      IPersistFile* ppf;
      hres = psl->QueryInterface(IID_IPersistFile, (void**)(&ppf));
      if (SUCCEEDED(hres))
      {
         // Set the path to the shell link target.
         psl->SetPath(Application->ExeName.c_str());
         // Set the description of the shell link.
		 psl->SetDescription("QuickSilver Lite");
		 // Save the link via the IPersistFile::Save method.
         ppf->Save(LinkName, TRUE);
         // Release pointer to IPersistFile.
         ppf->Release();
      }
      // Release pointer to IShellLink.
	  psl->Release();
   }
}
//---------------------------------------------------------------------------

void __fastcall TSetupDlg2::SaveSetup()
{
   AnsiString name;
   bool tor, same = ProxiesSame();
   TStringList *cfg = new TStringList();
   TIniFile *ini = new TIniFile(AppIni());

   // save default message
   cfg->Add("Fcc: outbox");
   cfg->Add("Host: " + SMTPServer->Text);
   cfg->Add("From: " + EmailAddress->Text);
   cfg->Add("Chain: *,*,*; copies=2;");
   cfg->Add("To: ");
   cfg->Add("Subject: ");
   cfg->Add("");
   cfg->Add("~~");
   DB->SaveDefMessage(cfg);

   // save default usenet article
   cfg->Clear();
   cfg->Add("Fcc: outbox");
   cfg->Add("Host: " + SMTPServer->Text);
   cfg->Add("From: " + EmailAddress->Text);
   cfg->Add("Chain: *,*,dizum; copies=2;");
   cfg->Add("To: mail2news@dizum.com");
   cfg->Add("Newsgroups: ");
   cfg->Add("Subject: ");
   cfg->Add("");
   cfg->Add("~~");
   DB->SaveDefArticle(cfg);

   // save bug report
   cfg->Clear();
   cfg->Add("########################################## ");
   cfg->Add("# ");
   cfg->Add("# THIS IS NOT AN ANONYMOUS MESSAGE ");
   cfg->Add("# ");
   cfg->Add("# If you report a bug I may very well need to communicate with ");
   cfg->Add("# you. People seldom describe their problems clearly enough that ");
   cfg->Add("# I can reproduce them without further discussion. I'd appreciate");
   cfg->Add("# an email address. Add a chain header if you must but chances ");
   cfg->Add("# are your problem may not get fixed. ");
   cfg->Add("# ");
   cfg->Add("Fcc: outbox");
   cfg->Add("Host: " + SMTPServer->Text);
   cfg->Add("From: " + EmailAddress->Text);
   cfg->Add("To: bugs@quicksilvermail.net");
   cfg->Add("Subject: QSLite Bug Report");
   cfg->Add("");
   cfg->Add("QSL version: ");
   cfg->Add("Operating System: ");
   cfg->Add("Linux/Wine Version: ");
   cfg->Add("");
   cfg->Add("Description of the problem and instuctions on how to reproduce it: ");
   cfg->Add("");
   cfg->Add("");
   cfg->Add("~~");
   DB->SaveTemplate("bug report", cfg);

   // save anon reply
   cfg->Clear();
   cfg->Add("Fcc: outbox");
   cfg->Add("Host: " + SMTPServer->Text);
   cfg->Add("From: " + EmailAddress->Text);
   cfg->Add("Chain: *,*,*; copies=2;");
   cfg->Add("To: ");
   cfg->Add("Subject: ");
   cfg->Add("");
   cfg->Add("~~");
   DB->SaveTemplate("anon reply", cfg);

   // save anon follow-up
   cfg->Clear();
   cfg->Add("Fcc: outbox");
   cfg->Add("Host: " + SMTPServer->Text);
   cfg->Add("From: " + EmailAddress->Text);
   cfg->Add("Chain: *,*,dizum; copies=2;");
   cfg->Add("To: mail2news@dizum.com");
   cfg->Add("References: ");
   cfg->Add("Newsgroups: ");
   cfg->Add("Subject: ");
   cfg->Add("");
   cfg->Add("~~");
   DB->SaveTemplate("anon follow-up", cfg);

   // smtp host
   cfg->Clear();
   cfg->Add("key=" + SMTPServer->Text);
   cfg->Add("host=" + SMTPServer->Text);
   cfg->Add("port=" + SMTPPort->Text);
   cfg->Add("tls=" + SMTPTls->Text);
   cfg->Add("auth=" + IntToStr(SMTPAuth->Checked));
   cfg->Add("userid=" + SMTPUserID->Text);
   cfg->Add("password=" + SMTPPassword->Text);
   DB->SaveSMTP(SMTPServer->Text, cfg);

   // smtp proxy and maybe http proxy
   cfg->Clear();
   if (SMTPProxyBox->Checked)
   {
	  tor = SProxyTOR->Checked;
	  if (same)
	  {
		 if (tor) name = "TOR";
		 else name = "SOCKS";
	  }
	  else
		 name = "SMTP";

	  cfg->Add("proxy=" + name);
	  cfg->Add("proto=" + AnsiString(same ? "sh" : "s"));
      cfg->Add("host=" + SProxyServer->Text);
	  cfg->Add("smtpproxy=1");
	  cfg->Add("smtpport=" + SProxyPort->Text);
	  if (same)
      {
         cfg->Add("httpproxy=1");
		 cfg->Add("httpport=" + HProxyPort->Text);
      }
      cfg->Add("socks=" + SProxySocks->Text);
      cfg->Add("tor=" + IntToStr(tor));
      cfg->Add("auth=" + IntToStr(SProxyAuth->Checked));
      cfg->Add("userid=" + SProxyUserID->Text);
      cfg->Add("password=" + SProxyUserID->Text);
      ini->WriteString("settings", "smtpproxy", name);
      if (same) ini->WriteString("settings", "httpproxy", name);
      DB->SaveProxy(name, cfg);
   }

   // http proxy if not same as smtp
   cfg->Clear();
   if (!same && HTTPProxyBox->Checked)
   {
      cfg->Add("proxy=HTTP");
      cfg->Add("proto=h");
      cfg->Add("host=" + HProxyServer->Text);
      cfg->Add("httpproxy=1");
      cfg->Add("httpport=" + HProxyPort->Text);
      cfg->Add("socks=" + HProxySocks->Text);
      cfg->Add("tor=" + IntToStr(HProxyTOR->Checked));
      cfg->Add("auth=" + IntToStr(HProxyAuth->Checked));
      cfg->Add("userid=" + HProxyUserID->Text);
      cfg->Add("password=" + HProxyUserID->Text);
      ini->WriteString("settings", "httpproxy", name);
	  DB->SaveProxy("HTTP", cfg);
   }

   if (CreateShortcut->Checked)
      CreateDesktopShortcut();
   delete ini;
   delete cfg;
}
//---------------------------------------------------------------------------

