/*
*  smtpdlgs.cpp
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <dateutils.hpp>
#include "smtpdlgs.h"
#include "mixmaster.h"
#include "utility.h"
#include "qsdbs.h"
#include "prevdlgs.h"
#include "eraser.h"
#include "mimeencs.h"
#include "klendlgs.h"
#include "global.h"
#include "gpg.h"
#include "openssl.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "OverbyteIcsSmtpProt"
#pragma link "OverbyteIcsWndControl"
#pragma link "OverbyteIcsWSocket"
#pragma link "OverbyteIcsLogger"
#pragma resource "*.dfm"
TSMTPDlg *SMTPDlg = NULL;
const AnsiString Success = "All mail sent!";
const AnsiString badreplycodes = "550 ";
const AnsiString DefTORSignOn = "example.com"; // or inifile 'settings|torsignon'
//---------------------------------------------------------------------------

bool __fastcall SendMail(TForm *Owner)
{
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)" Sending...");
   SMTPDlg = new TSMTPDlg(Owner, fsNormal);
   SMTPDlg->ShowModal();
   bool ok = SMTPDlg->Display->Text.Pos(Success) > 0;
   delete SMTPDlg;
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"");
   return ok;
}
//---------------------------------------------------------------------------
#ifdef QS2
bool __fastcall SendNewsreaderPost(TForm *Owner)
{
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)" Sending...");
   SMTPDlg = new TSMTPDlg(Owner, fsNormal);
   SMTPDlg->ShowModal();
   bool ok = SMTPDlg->Display->Text.Pos(Success) > 0;
   delete SMTPDlg;
   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"");
   return ok;
}
#endif
//---------------------------------------------------------------------------

__fastcall TSMTPDlg::TSMTPDlg(TComponent* Owner)
		: TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TSMTPDlg::TSMTPDlg(TComponent* Owner, TFormStyle FormStyle)
		: TForm(Owner)
{
   this->FormStyle = FormStyle;
   PopupParent = (TForm*)Owner;
   Display->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   preview = ini->ReadBool("settings", "previewsend", false);
   LoadWindowPosition(this, "smtpdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Display->Font, ini->ReadString("settings", "editorfont", OptsDefEditorFont));
   SslContext->SslCipherList = ini->ReadString(SSLCipherSuites, SmtpSSLciphersuite, SmtpSSLCipherSuite);
   defproxy = ini->ReadString("settings", "smtpproxy", "none");
   usesslcert = ini->ReadBool("settings", "usesslcert", false);
   subranddate = ini->ReadBool("settings", "subrandomdate", false);
   tlspreferred = ini->ReadBool("settings", "tlspreferred", false);
   bool ssllog = ini->ReadBool("settings", "ssllog", false);
   if (ssllog)
   {
	   bool append = ini->ReadBool("settings", "ssllogappend", false);
	   IcsLogger->LogFileOption = append ? lfoAppend : lfoOverwrite;
	   IcsLogger->LogFileName = AppDataDir()+"ssllog.txt";
	   SMTP->CtrlSocket->IcsLogger = IcsLogger;
	   SslContext->IcsLogger = IcsLogger;
   }
#ifndef QS2
   expertmode = ini->ReadBool("settings", "expertmode", false);
#else
   expertmode = true;
#endif
   closesmtp = ini->ReadBool("settings", "closesmtp", false);
   delete ini;

   orgcap = "";
   display = Display->Lines;

   feedback = new TStringList();
   mpackets = new TStringList();
   messages = new TStringList();
   message = new TStringList();
   msg = new TMsgManager(message);
   proxy = new TStringList();
   mixheaders = new TStringList();
   verbose = new TStringList();
   brief = new TStringList();
   hosts = new TStringList();
   mxhosts = new TStringList();
   nodirect = new TStringList();
   DB->LoadBars(BarDirect, nodirect);
   if (FileExists(MixMixMxFile()))
	  mxhosts->LoadFromFile(MixMixMxFile());
   shortlookup = new TStringList();
   // load email=shortname list
   if (FileExists(MixPubringMixFile()))
   {
	  TStringList *pubring = new TStringList();
	  pubring->LoadFromFile(MixPubringMixFile());
	  for (int i = 0; i < pubring->Count; i++)
	  {
		 AnsiString line = pubring->Strings[i];
		 if (line.Pos("@") > 0)
			shortlookup->Add(GetTok(1, line, ' ')+"="+GetTok(0, line, ' '));
	  }
	  delete pubring;
   }
   ProgressBar->DoubleBuffered = true;
   //ProgressBar->Top = CancelBtn->Top + 2;
   BytesSent = BytesTotal = AllSent = 0;
   failed = canceled = cancelhandled = connecting = false;
   msgcount = mixcount = 0;
   lasterror = 0;
   mixqueue = NULL;
   curprobs = DB->QueueCount(ProbQueue);
   //SetEditRect(Display);
   Timer2->Enabled = true;
}
//---------------------------------------------------------------------------

__fastcall TSMTPDlg::~TSMTPDlg()
{
   if (IntToStr(Width)+IntToStr(Height) != orgwin)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  SaveWindowPosition(this, "smtpdlg", ini);
	  delete ini;
   }
   delete shortlookup;
   delete nodirect;
   delete mxhosts;
   delete hosts;
   delete brief;
   delete verbose;
   delete mixheaders;
   delete proxy;
   delete msg;
   delete message;
   delete messages;
   delete mpackets;
   delete feedback;
   SMTPDlg = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (!expertmode && DB->QueueCount(ProbQueue))
   {
	  ((TForm*)Owner)->Perform(WM_LOADPROBLEM, 0, 0);
   }
   if (!expertmode && DB->QueueCount(MailQueue))
   {
	  ((TForm*)Owner)->Perform(WM_LOADCANCEL, 0, 0);
   }
   MixClearMixmasterPool();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Timer2Timer(TObject *Sender)
{
   Timer2->Enabled = false;
   MixClearMixmasterPool();
   SendMail();
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::SendMail()
{
   DB->LoadQueueIndex(MailQueue, messages);
   msgcount = messages->Count;
   assert(msgcount);
   goods = 0;

   assert(defproxy != "");
   if (defproxy != "none")
   {
	  if (!DB->ProxyExists(defproxy))
	  {
		 Snooze(200);
		 MsgBox(this,
			"The default SMTP proxy you've been using, " + defproxy + ", is missing!  \n"
			"You currently don't have an assigned proxy. ",
			MT_ERROR, MB_OK);
		 defproxy = "none";
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteString("settings", "smtpproxy", defproxy);
		 delete ini;
		 return false;
	  }
   }

   Log("Session: " + FormatDateTime("yyyy-mm-dd hh:nn:ss", Now()));
   Log("Default proxy: " + defproxy);
   Log("");

   PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   SendNextMessage();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TSMTPDlg::GetErrorStr(int errcode)
{
   AnsiString errmsg = WSocketErrorDesc(errcode);
   if (errmsg.Pos("Not a WinSock error") > 0)
	  errmsg = SMTP->ErrorMessage;
   if (errmsg.IsEmpty())
      errmsg = "Unknown error";
   AnsiString errcodestr = IntToStr(errcode);
   if (errmsg.Pos(errcodestr) == 1)
	  errmsg = errmsg.SubString(errcodestr.Length() + 2, errmsg.Length());
   return " " + errmsg;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2Display(TObject *Sender, AnsiString Msg)
{
   if (connecting)
   {
	  LogLast("Connected to " + SMTP->Host + ":" + SMTP->Port);
	  connecting = false;
	  inremmsg = false;
   }

   if (Msg.Pos("> ") == 1 || Msg.Pos("< ") == 1)
   {
	  if (Msg.Pos(MixEndRemailerMessage) > 0)
		 inremmsg = false;

	  if (!inremmsg)
		 verbose->Add(Msg);

	  if (Msg.Pos(MixBeginRemailerMessage) > 0)
	  {
		 inremmsg = true;
		 verbose->Add("> ...");
	  }
   }

   if (Msg.SubString(1,1) == "<")
   {
	  int x; while ((x = Msg.Pos("  ")) > 0)
		 Msg.Delete(x, 1);
	  Msg = Msg.SubString(2, Msg.Length());
	  Log(Msg); 
	  AnsiString resultcode = Msg.SubString(2, 4);
	  if (badreplycodes.Pos(resultcode) > 0)
	  {
		 Log(" ERROR: Bad result code " + resultcode);
		 lasterror = resultcode.ToIntDef(1000);
	  }
	  else
	  {
		 //if (SMTP->Host == "mail.hermetix.org" && Msg.Pos("221 2.0.0 Bye") > 0)
		 //	SMTP->CtrlSocket->Close();
		 if (Msg.TrimLeft().Pos("221") == 1)// 2.0.0") > 0)
			SMTP->CtrlSocket->Close();
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTPResponse(TObject *Sender, AnsiString Msg)
{
   if (Msg.Pos("250 STARTTLS") == 1 || Msg.Pos("250-STARTTLS") == 1)
	  SMTP->SslType = smtpTlsExplicit;
   //if (Msg.Pos("220 TLS go ahead"))
   //  SMTP2RequestDone(SMTP, smtpStartTls, 0);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTPCommand(TObject *Sender, AnsiString Msg)
{
   //
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2GetData(TObject *Sender, int LineNum,
	  Pointer MsgLine, int MaxLen, bool &More)
{
   ProgressBar->Position = (BytesSent/BytesTotal) * 100;
   if (LineNum > msg->body->Count || Cancel())
   {
	  strcpy((char*)MsgLine,"");
	  More = false;
	  return;
   }
//sm(IntToStr(msg->body->Count)+"   "+IntToStr(LineNum)+"   "+IntToStr(MaxLen));

   AnsiString line = msg->body->Strings[LineNum - 1];
   int len = line.Length() + 2;
   strcpy((char*)MsgLine, line.c_str());
   BytesSent += len;
   AllSent += len;
   More = true;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2ProcessHeader(TObject *Sender,
	  TStrings *HdrLines)
{
   HdrLines->Clear();
   for (int i = 0; i < msg->headers->Count; i++)
	  HdrLines->Add(msg->headers->Strings[i]);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2RequestDone(TObject *Sender,
	  TSmtpRequest RqType, WORD ErrCode)
{
   if (Cancel())
	  return;

   if (ErrCode)
   {
	  failed = true;
	  lasterror = ErrCode;
	  Log(" ERROR: " + AnsiString(ErrCode) + GetErrorStr(ErrCode));
	  Log(" Error at Request Done ");
	  SMTP->Abort();
	  PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
	  return;
   }

   switch (RqType)
   {
	  case smtpOpen:
		 SMTP->MailFrom();
		 break;
	  case smtpConnect:
		 if (SMTP->AuthType == smtpAuthNone)
			SMTP->Ehlo();
		 else
			SMTP->Ehlo();
		 break;
	  case smtpHelo:
		 SMTP->MailFrom();
		 break;
	  case smtpEhlo:
		 if (SMTP->SslType == smtpTlsExplicit)
		 {
			FEhloCount++;
			if (FEhloCount == 1)
			{
			   SMTP->StartTls();
			}
			else
			{
			   if (SMTP->AuthType == smtpAuthNone)
				  SMTP->MailFrom();
			   else
				  SMTP->Auth();
			}
		 }
		 else
		 {
		   if (SMTP->AuthType == smtpAuthNone)
			  SMTP->MailFrom();
		   else
			  SMTP->Auth();
		 }
		 break;
	  case smtpStartTls:
		 SMTP->Ehlo(); // We need to re-issue the Ehlo command
		 break;
	  case smtpAuth:
		 SMTP->MailFrom();
		 break;
	  case smtpMailFrom:
		 SMTP->RcptTo();
		 break;
	  case smtpRcptTo:
		 SMTP->Data();
		 break;
	  case smtpData:
		 SMTP->Quit();
		 break;
	  case smtpQuit:
		 success = true;
		 goods++;
#ifdef QS2
		 if (!currinreplyto.IsEmpty())
		 {
			Application->MainForm->Perform(WM_FLAGMESSAGE, mflReplied, (LPARAM)currinreplyto.c_str());
			currinreplyto = "";
		 }
#endif
		 PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
		 break;
   }
}
//---------------------------------------------------------------------------
// fix for connection reset by peer error in sessionclosed when mail actually sent
bool __fastcall TSMTPDlg::CheckTrancriptForSuccess()
{
   int c = feedback->Count;
   if (c < 3) return false;
   if (feedback->Strings[c - 1].TrimLeft().Pos("221") == 1)
   {
	  if (feedback->Strings[c - 2].TrimLeft().Pos("250") == 1)
	  {
		 if (feedback->Strings[c - 3].TrimLeft().Pos("354") == 1)
			return true;
	  }
   }
   return false;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2SessionClosed(TObject *Sender, WORD ErrCode)
{
   if (ErrCode)
   {
	  /*failed = !CheckTrancriptForSuccess();
	  if (!failed)
	  {
		 //Log("Connection closed!", true);
		 success = true;
		 goods++;
		 //return;
	  }
	  else*/
	  {
		 lasterror = ErrCode;
		 Log(" ERROR: " + AnsiString(ErrCode) + " " + GetErrorStr(ErrCode), true);
		 Log(" Error at Session Closed ", true);
		 Log("Connection closed!", true);
		 SMTP->Abort();
		 PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
	  }
	  return;
   }

   Log("Connection closed!", true);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTP2SessionConnected(TObject *Sender, WORD ErrCode)
{
   connecting = false;
   if (ErrCode)
   {
	  failed = true;
	  lasterror = ErrCode;
	  Log(" ERROR: " + AnsiString(ErrCode) + " " + GetErrorStr(ErrCode), true);
	  Log(" Error at Session Connected ", true);
	  SMTP->Abort();
      PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
	  return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTPRcptToError(TObject *Sender, WORD ErrCode,
	  int RcptNameIdx, TSmtpRcptToErrorAction &Action)
{
   Action = srteAbort;
   failed = true;
   lasterror = ErrCode;
   Log(" ERROR: " + AnsiString(ErrCode) + " " + GetErrorStr(ErrCode), true);
   Log(" Error at RCPT TO ", true);
   SMTP->Abort();
   PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SMTPMessageDataSent(TObject *Sender, int Size)
{
   //
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::ShowExitInfo()
{
   if (showedexitinfo)
	  return;

   showedexitinfo = true;
   brief->Text = Display->Text;
   if (mixqueue)
   {
	  delete mixqueue;
	  mixqueue = NULL;
   }

   while (SMTP->Connected)
	  Application->ProcessMessages();

   int m = DB->QueueCount(MailQueue);
   int p = DB->QueueCount(ProbQueue);
   if (expertmode)
   {
	  AnsiString probsmsg = IntToStr(p) + " in problem queue ";
	  if (curprobs)
		 probsmsg += "(" + IntToStr(p - curprobs) + " new) ";
	  Log(IntToStr(m) + " in message queue ", true);
	  Log(probsmsg, true);
   }
   Log(IntToStr(goods) + " sent ", true);
   Log("");

#ifdef QS2
   ShowStatus((curprobs ? "ERROR sending, click here" : "send complete"));
#endif

   bool doclose = false;
   if (!m && p == curprobs)
   {
	  Log(Success, true); // "All mail sent!"
	  if (expertmode && closesmtp)
		 doclose = true;
   }

   ActiveControl = Panel1;
   CancelBtn->Caption = "Close";
   ProgressBar->Hide();
   Verbose->Show();
   SaveSessionLog();

   Snooze(500);

   if (doclose)
	  ModalResult = mrOk;

#ifdef QS2
   ((TForm*)Owner)->Perform(WM_TAPQUEUES, 0, 0);
#endif
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SetupProxy()
{
   SMTP->CtrlSocket->SocksServer = "";
   SMTP->CtrlSocket->SocksPort = "";
   SMTP->CtrlSocket->SocksLevel = "4a";
   SMTP->CtrlSocket->SocksUsercode = "";
   SMTP->CtrlSocket->SocksPassword = "";
   SMTP->CtrlSocket->SocksAuthentication = socksNoAuthentication;

   if (proxyinuse != "none")
   {
	  DB->LoadProxy(proxyinuse, proxy);
	  assert(proxy->Count > 0);
	  assert(proxy->Values["host"] != "");
	  assert(proxy->Values["smtpport"] != "");
	  assert(proxy->Values["socks"] != "" && proxy->Values["socks"] != "no");
	  SMTP->CtrlSocket->SocksServer = proxy->Values["host"];
	  SMTP->CtrlSocket->SocksPort = proxy->Values["smtpport"];
	  SMTP->CtrlSocket->SocksLevel = proxy->Values["socks"];

	  if (ReadBool(proxy, "auth", false))
	  {
		 assert(proxy->Values["password"] != "");
		 assert(proxy->Values["password"] != "");
		 SMTP->CtrlSocket->SocksPassword = proxy->Values["password"];
		 SMTP->CtrlSocket->SocksUsercode = proxy->Values["userid"];
		 SMTP->CtrlSocket->SocksAuthentication = socksAuthenticateUsercode;
	  }
   }
}
//---------------------------------------------------------------------------

int __fastcall prioritycompare(TStringList *list, int index1, int index2)
{
   int val1 = GetTok(mxPriority, list->Strings[index1], ',').ToIntDef(0);
   int val2 = GetTok(mxPriority, list->Strings[index2], ',').ToIntDef(0);
   if (val1 < val2) return -1;
   if (val1 > val2) return 1;
   return 0;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::GetHosts(TMsgManager *msg, TStringList *Hosts)
{
   Hosts->Clear();
   AnsiString hoststr, email = msg->to, rem = shortlookup->Values[email];
   if (rem != "" && nodirect->IndexOf(rem) < 0)
   {
	  TStringList *mix = new TStringList();
	  DB->LoadConfig(MIXConfig, mix);
	  bool direct = ReadBool(mix, "mxdirect", false);
	  delete mix;

	  if (direct)
	  {
		 for (int i = 0; i < mxhosts->Count; i++)
		 {
			hoststr = mxhosts->Strings[i];
			if (hoststr.Pos(rem+"=") == 1)
			   Hosts->Add(hoststr.SubString(rem.Length()+2, hoststr.Length()));
		 }
	  }
   }

   Hosts->CustomSort(prioritycompare);
   if (Hosts->IndexOf(msg->host) < 0)
	  Hosts->Add(msg->host);
   if (hosts->Count > 1)
   {
	  Log("Hosts available:");
	  for (int i = 0; i < hosts->Count; i++)
		 Log(" " + GetTok(0, hosts->Strings[i], ','));
	  Log("");
   }
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::GetHost(bool TOR, TStrings *Hosts, TStrings *cfg)
{
   assert(Hosts->Count);
   AnsiString hoststr, host, port, hst, prt;

   bool hdrpreferred = false;
   if (tlspreferred)
   {
	  host = GetTok(0, Hosts->Strings[Hosts->Count-1], ':');
	  DB->LoadSMTP(host, cfg);
	  if (Hosts->Count == 1)
	  {
		 Hosts->Delete(0);
		 return;
	  }
	  AnsiString tls = ReadString(cfg, "tls", "none").LowerCase();
	  hdrpreferred = tls == "tls" || tls == "ssl";
   }

startagain:

   while (!TOR && Hosts->Count && Hosts->Strings[0].Pos(".onion,") > 0)
	  Hosts->Delete(0);

   assert(Hosts->Count);
   hoststr = Hosts->Strings[0];
   host = GetTok(mxHost, hoststr, ',');
   Hosts->Delete(0);
   DB->LoadSMTP(host, cfg);
   if (!Hosts->Count && !cfg->Count)
   {
	  hst = GetTok(0, host, ':'); prt = GetTok(1, host, ':');
	  if (prt.IsEmpty())
		 prt = "25";
	  cfg->Add("host=" + hst);
	  cfg->Add("port=" + prt);
	  cfg->Add("tls=none");
   }
   if (cfg->Count)
	  return;

   if (GetTok(mx587, hoststr, ',').LowerCase() == "yes")
   {
	  cfg->Add("host=" + host);
	  cfg->Add("port=587");
	  cfg->Add("tls=TLS");
	  return;
   }

   if (GetTok(mx465, hoststr, ',').LowerCase() == "yes")
   {
	  cfg->Add("host=" + host);
	  cfg->Add("port=465");
	  cfg->Add("tls=SSL");
	  return;
   }

   if (GetTok(mx2525, hoststr, ',').LowerCase() == "yes")
   {
	  cfg->Add("host=" + host);
	  cfg->Add("port=2525");
	  cfg->Add("tls=TLS");
	  return;
   }

   if (GetTok(mxTLS, hoststr, ',').LowerCase() == "yes")
   {
	  cfg->Add("host=" + host);
	  cfg->Add("port=25");
	  cfg->Add("tls=TLS");
	  return;
   }

   if (hdrpreferred)
   {
	  goto startagain;
   }

   cfg->Clear();
   hst = GetTok(0, host, ':'); prt = GetTok(1, host, ':');
   if (prt.IsEmpty())
	  prt = "25";
   cfg->Add("host=" + hst);
   cfg->Add("port=" + prt);
   cfg->Add("tls=none");
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::PrepareAndConnect()
{
   success = false;

   if (Cancel())
	  return false;

   while (message->Count && message->Strings[0].Pos("#") == 1)
	  message->Delete(0);

   SetupProxy();
   bool TOR = proxyinuse != "none" && ReadBool(proxy, "tor", false);
   SMTP->AuthType = smtpAuthNone;

   TStringList *cfg = new TStringList();
   GetHost(TOR, hosts, cfg);

   SMTP->Host = cfg->Values["host"];
   SMTP->Port = cfg->Values["port"];
   AnsiString tls = cfg->Values["tls"].LowerCase();

   SMTP->SslType = tls == "tls" ? smtpTlsExplicit : tls == "ssl" ?
	  smtpTlsImplicit : smtpTlsNone; // if SslType in not set to Explicit and QS
									 // see '250-STARTTLS' it turns on Explicit
   if (ReadBool(cfg, "auth", false))
   {
	  assert(cfg->Values["password"] != "");
	  assert(cfg->Values["userid"] != "");
	  SMTP->Password = cfg->Values["password"];
	  SMTP->Username = cfg->Values["userid"];
	  SMTP->AuthType = smtpAuthLogin;
   }
   delete cfg;

   BytesSent = 0; BytesTotal = msg->body->Length();
   if (usesslcert && !CheckCreateQSPem(this, SslContext))
	  return false;

   TIniFile *ini = new TIniFile(AppIni());

   SMTP->FromName = msg->GetAddress(msg->from);
   AnsiString HELO, SignOn;
   if (proxyinuse != "none" && TOR)
	  HELO = ini->ReadString("settings", "torsignon", DefTORSignOn);
   if (HELO.IsEmpty())
	  HELO = GetDomain(SMTP->FromName);
   SignOn = msg->GetHeader("Sign-On: ");
   if (!SignOn.IsEmpty())
	  HELO = SignOn;
   SMTP->SignOn = HELO;

   delete ini;

   SMTP->HdrFrom = msg->from;
   SMTP->HdrSubject = msg->subject;
   SMTP->HdrTo = msg->to;
   msg->GetRecipients(SMTP->RcptName);
   if (subranddate)
   {
	  TDateTime date = Now();
	  //Randomize(); this is called when program starts
	  date = IncSecond(date, -Random(4 * 24 * 60 * 60));
	  msg->AppHeader("Date: " + Rfc822DateTime(date));
   }
//look(msg->Lines, "before del proxy");
   msg->DelHeader("Sign-On:");
   msg->DelHeader("Host:");
   msg->DelHeader("Bcc:");
   msg->DelHeader("Fcc:");
   msg->DelHeader("PgpBtn:");
   msg->DelHeader("Pgp:");
   msg->DelHeader("Proxy:");
//look(msg->Lines, "after del proxy");

   //msg->AppHeader("Lines: 41");
   //look(msg->Lines);

   FEhloCount = 0;
   lasterror = 0;
   lasterrorstr = "";
   feedback->Clear();

   if (Cancel())
	  return false;

   Log("Connecting to " + SMTP->Host + ":" + SMTP->Port);
   Snooze(500);
   connecting = true;
   SMTP->Connect();
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::PrepareAndConnectNextHost()
{
   success = false;

   if (Cancel())
	  return false;

   if (!hosts->Count)
	  return false;

   SMTP->Abort();
   while (SMTP->Connected)
	  Application->ProcessMessages();

   Log("Next host...");

   SetupProxy();
   bool TOR = proxyinuse != "none" && ReadBool(proxy, "tor", false);
   SMTP->AuthType = smtpAuthNone;

   TStringList *cfg = new TStringList();
   GetHost(TOR, hosts, cfg);
   SMTP->Host = cfg->Values["host"];
   SMTP->Port = cfg->Values["port"];
   AnsiString tls = cfg->Values["tls"].LowerCase();
   SMTP->SslType = tls == "tls" ? smtpTlsExplicit : tls == "ssl" ?
	  smtpTlsImplicit : smtpTlsNone;
   if (ReadBool(cfg, "auth", false))
   {
	  assert(cfg->Values["password"] != "");
	  assert(cfg->Values["userid"] != "");
	  SMTP->Password = cfg->Values["password"];
	  SMTP->Username = cfg->Values["userid"];
	  SMTP->AuthType = smtpAuthLogin;
   }
   delete cfg;

   BytesSent = 0; BytesTotal = msg->body->Length();
   if (usesslcert && !CheckCreateQSPem(this, SslContext))
	  return false;

   FEhloCount = 0;
   lasterror = 0;
   Log("Connecting to " + SMTP->Host);
   connecting = true;
   feedback->Clear();

   while (SMTP->Connected)
	  Application->ProcessMessages();

   SMTP->Connect();
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::SendNextMessage()
{
   if (Cancel())
	  return false;

   while (SMTP->Connected)
	  Application->ProcessMessages();

   if (lasterror)
   {
	  Log("Failed");
	  Log("");
	  if (PrepareAndConnectNextHost())
		 return true;

	  HandleFailedMessageFeedback();
	  ShowExitInfo();
	  return false;
   }

   if (success)
   {
	  Log("Ok");
	  Log("");
   }

   success = false;

   Snooze(500);
   ProgressBar->Position = 0;
   ProgressBar->Max = 100;
   ProgressBar->Show();
   Snooze(10);

   feedback->Clear();
   showedexitinfo = false;

   if (!mpackets->Count && !messages->Count)
   {
	  UnqueueAllAndFcc();
	  ShowExitInfo();
	  if (autoclose)
	  {
		 Snooze(1000);
		 CancelBtn->Click();
		 Snooze(1000);
		 ((TForm*)Owner)->Close();
	  }
	  return false;
   }

   if (mpackets->Count) // first handle mixmaster encrypted messages, if any
   {
	  UnqueueMixmaster(mpackets->Strings[0], message);
	  laststatusmessage = "Sending mixmaster packet " + IntToStr(mixcount - (mpackets->Count - 1)) +
		" of " + IntToStr(mixcount) + "  ";
	  Log(laststatusmessage);
#ifdef QS2
	  ShowStatus(laststatusmessage.LowerCase());
#endif;

	  // insert headers (host, sign-on and from) saved in earlier pass (below)
	  message->Delete(0); // 0 is From: header added by mix
	  for (int i = 0; i < mixheaders->Count; i++)
		 message->Insert(i, mixheaders->Strings[i]);
	  // remove packet from list
	  mpackets->Delete(0);
	  if (!mpackets->Count)
	  {
		 delete mixqueue;
		 mixqueue = NULL;
		 mixcount = 0;
	  }
   }
   else // handle mail from message queue
   {
	  if (!messagekey.IsEmpty())
		 UnqueueAllAndFcc();

	  mixheaders->Clear();
	  messagekey = messages->Strings[0];
	  DB->UnQueueMessage(MailQueue, messagekey, message);
	  DB->QueueMessage(ProbQueue, messagekey, message);
	  laststatusmessage = "Sending message " + IntToStr(msgcount - (messages->Count - 1)) +
		 " of " + IntToStr(msgcount);
	  Log(laststatusmessage);
#ifdef QS2
	  ShowStatus(laststatusmessage.LowerCase());
	((TForm*)Owner)->Perform(WM_TAPQUEUES, 0, 0);
#endif
	  messages->Delete(0);
	  if (!messages->Count)
		 msgcount = 0;
   }
#ifdef QS2
   currinreplyto = msg->GetHeader("In-Reply-To:");
   if (!currinreplyto.IsEmpty())
	  msg->DelHeader("In-Reply-To:");
#endif
   AnsiString hdrproxy = msg->GetHeader("Proxy:");
   proxyinuse = hdrproxy.IsEmpty() ? defproxy : hdrproxy;
   assert(proxyinuse != "");
   if (proxyinuse != "none" && !DB->ProxyExists(proxyinuse))
   {
	  if (proxyinuse == hdrproxy)
	  {
		 MsgBox(this,
			"The proxy you've assigned in your Proxy header, " + proxyinuse + ", is missing! Your message \n"
			"is in the Problem Queue. You'll need to change to another proxy.",
			MT_INFO, MB_OK);
		 Log("Send aborted");
		 ShowExitInfo();
		 return false;
	  }
	  else
	  {
		 MsgBox(this,
			"The default proxy you have selected, " + proxyinuse + ", is missing. Your message  \n"
			"is in the Problem Queue. Currently you don't have a proxy assigned!",
			 MT_INFO, MB_OK);
		 Log("Send aborted");
		 proxyinuse = "none";
		 TIniFile *ini = new TIniFile(AppIni());
		 ini->WriteString("settings", "smtpproxy", proxyinuse);
		 delete ini;
		 ShowExitInfo();
		 return false;
	  }
   }
   Log("Proxy: " + proxyinuse);

   if (msg->IsPost())
	  MassageReferences(message);

   if (Cancel())
	  return false;

   // handled in QueueMessage -- composewins.cpp
   /*TIniFile *ini = new TIniFile(AppIni());
   int wrapat = ini->ReadInteger("settings", "wordwraponsend", OptsDefMessageWrap);
   AnsiString qm = ini->ReadString("settings", "quotemarks", OptsDefQuoteMarks);
   delete ini;

   bool finalformat = msg->GetHeader("Final-Format:") == "yes";
   if (!finalformat &&
	  message->IndexOf(BeginREMAILERMessage) < 0 &&
	  message->IndexOf(BeginPGPMessage) < 0 &&
	  message->IndexOf(BeginPGPSignedMessage) < 0)
   {
	  //msg->WrapMessage(wrapat, qm);
	  //msg->CutMessage();
   }
   else
	  msg->DelHeader("Final-Format:");*/

   if (HasAttachments(message) && !EncodeMIMEAttachments(this, message, false))
   {
	  MsgBox(this,
		"Sorry, I've failed to MIME encode your attachments! ",
		 MT_ERROR, MB_OK);
	  ShowExitInfo();
	  return false;
   }

   // previewed in composewins.cpp
   /*if (preview &&
	  message->IndexOf(BeginREMAILERMessage) < 0 &&
	  message->IndexOf(BeginPGPMessage) < 0)
   {
	  TIniFile *ini = new TIniFile(AppIni());
	  int wrapat = ini->ReadInteger("settings", "wordwraponsend", OptsDefMessageWrap);
	  delete ini;

	  if (PreviewMessage(this,
		 "message body wrapped at " + IntToStr(wrapat) + " columns",
		  message) == IDCANCEL)
	  {
		 //Log("User canceled operation!");
		 canceled = true;
		 Cancel();
		 ShowExitInfo();
		 return false;
	  }
	  Snooze(300);
   }*/

   if (Cancel())
	  return false;

   if (msg->HasHeader("Chain:"))
   {
	  Log(" Encountered mixmaster message ");
	  // save headers for later adding to mix messages just before send
	  msg->GetHeaders("Host:,Sign-On:,From:,Proxy:", mixheaders);
	  bool usercancel, ok;

	  ok = EncryptMessage(message, usercancel);

	  Snooze(100);

	  if (Cancel())
		 return false;

	  if (ok)
	  {
		 mpackets->Text = message->Text;
		 mixcount = mpackets->Count;
	  }
	  else
	  {
		 canceled = usercancel;
		 if (canceled)
			Cancel();
		 ShowExitInfo();
		 return false;
	  }

	  PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
	  return true;
   }

   assert(!msg->HasHeader("Chain:"));
   msg->DelHeader("Proxy:");

   if (preview)
   {
	  if (PreviewMessage(this, "message ready to send", message) == IDCANCEL)
	  {
		 canceled = true;
		 Cancel();
		 ShowExitInfo();
		 return false;
	  }
   }

   if (Cancel())
	  return false;

   if (DirectoryExists(msg->host))
   {
	  if (!SendMessageToDirectory(message, msg->host))
	  {
		 canceled = true;
		 Cancel();
		 ShowExitInfo();
		 return false;
	  }

	  PostMessage(Handle, WM_SENDNEXTMESSAGE, 0, 0);
	  return true;
   }

   GetHosts(msg, hosts);
   curhost = 0;
   Snooze(500);

   PrepareAndConnect();
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::HandleFailedMessageFeedback()
{
   DB->UnQueueMessage(OrigQueue, messagekey, message);
   // reformat feedback so lines are short enough they won't be wrapped
   feedback->Insert(0, "To: me@my.net");
   feedback->Insert(1, "");
   TMsgManager *man = new TMsgManager(feedback);
   man->WrapMessage(54, "");
   delete man;
   feedback->Delete(1);
   feedback->Delete(0);

   int idx = 0; // add problem feedback
   message->Insert(idx++,
	  "##########################################################");
   for (int i = 0; i < feedback->Count; i++)
	  message->Insert(idx++, "# " + feedback->Strings[i].Trim());
   message->Insert(idx, "# ");
   DB->QueueMessage(OrigQueue, messagekey, message);
#ifdef QS2
   ((TForm*)Owner)->Perform(WM_TAPQUEUES, 0, 0);
#endif
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::UnqueueAllAndFcc()
{
   DB->DelQueueMessage(ProbQueue, messagekey);
   DB->UnQueueMessage(OrigQueue, messagekey, message);
   AnsiString mailbox = msg->GetHeader("Fcc:");
   if (!mailbox.IsEmpty())
	  DB->SaveMessage(mailbox, message);
#ifdef QS2
   ((TForm*)Owner)->Perform(WM_TAPQUEUES, 0, 0);
#endif

}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::UnqueueMixmaster(const AnsiString &key, TStrings *message)
{
   mixqueue->GetText(key, message);
   assert(message->Count > 0);
   mixqueue->DelItem(key);
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::SendMessageToDirectory(TStrings *Message,
   AnsiString directory)
{
   assert(DirectoryExists(directory));
   if (directory[directory.Length()] != '\\')
	  directory += '\\';

   AnsiString filename, msgname;
   TIniFile *ini = new TIniFile(AppIni());
   msgname = ini->ReadString("settings", send2dirname, send2dirmsgname);
   delete ini;

   int x = 1;
   while (true)
   {
	  filename = directory+ChangeFileExt(msgname,"")+IntToStr(x)+ExtractFileExt(msgname);
	  if (!FileExists(filename))
		 break;
	  x++;
   }

   TMsgManager *msg = new TMsgManager(Message);
   msg->DelHeader("Host:");
   delete msg;

   try
   {
	  Message->SaveToFile(filename);
   }
   catch(Exception &e)
   {
	  MsgBox(this,
		 "I failed to save the message to the directory:  \n" +
		  directory + "\n\n"
		 "The exception reported was: \n" +
		  e.Message,
		  MT_ERROR, MB_OK);
	  Log("FAILED saving message: " + filename + "\r\n");
	  return false;
   }

   Log("OK! Message saved as: " + filename + "\r\n");
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::LoadMixmasterQueue(TStrings *filenames)
{
   assert(mixqueue == NULL);
   assert(filenames->Count > 0);

   mixqueue = new TQSDB("");
   TStringList *msg = new TStringList();
   for (int i = 0; i < filenames->Count; i++)
   {
	  AnsiString name = filenames->Strings[i];
      msg->LoadFromFile(name);
	  mixqueue->PutText(name, msg);
   }
   delete msg;
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::EncryptMessage(TStrings *message, bool &usercancel)
{
   TStringList *Feedback = new TStringList();
   TStringList *clrmessage = new TStringList();
   clrmessage->AddStrings(message);

   bool brok, dupl, ok;
   AnsiString brokreason, duplreason;
   int cbrok = 0, cdupl = 0, tries = 0;
   for (;;)
   {
	  Feedback->Clear();

	  ShowStatus(" Encrypting... ");
	  ok = MixEncryptMessage(this, message, Feedback, preview);
	  usercancel = !ok && Feedback->Text.Pos("User canceled operation!") == 1;
	  if (!usercancel)
	  {
		 LogMixmasterResults(ok, Feedback);
		 if (!ok)
			HandleFailedMessageFeedback();
	  }
	  ShowStatus("");

	  if (ok)
	  {
		 brok = MixHasBrokenChainAfterEncrypt(Feedback, brokreason);
		 dupl = MixHasDuplicateChainAfterEncrypt(Feedback, duplreason);

		 if (brok || dupl)
		 {
			//ClearMixmasterPool();
			if (tries == 12)
			{
			   ReportFailureOnDupOrBrokeChains(tries, cbrok, cdupl);
			   MixClearMixmasterPool();
			   delete clrmessage;
			   delete Feedback;
			   //aborted = true;
			   //ShowExitInfo();
			   return false;
			}

			if (Cancel())
			{
			   break;
			}

			Log(" I've encountered a problem with remailer selections, no big deal");
			if (brok)
			{
			   Log(" " + brokreason);
			   cbrok++;
			}
			if (dupl)
			{
			   Log(" " + duplreason);
			   cdupl++;
			}
			Log(" starting again...");
			Log("");

			message->Text = clrmessage->Text;
			ProgressBar->Hide();
			tries++;

			continue;
		 }

	  // message holds list of filenames from mix pool
		 LoadMixmasterQueue(message);
	  }
	  break;
   }

   MixClearMixmasterPool();
   delete clrmessage;
   delete Feedback;

// temp testing for unhandled mix messages
   TStringList *names = new TStringList();
   GetFileNames(names, MixPoolDir() + "*.*");
   assert(names->Count == 0);
   delete names;

   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::LogMixmasterResults(bool Success, TStrings *Feedback)
{
   assert(Feedback->Count > 0);
   if (Success)
   {
	  Log(" Mixmaster reports " + IntToStr(Feedback->Count) + " chain" +
		(Feedback->Count == 1 ? ": " : "s: "));
   }
   else
   {
	  Log(" Mixmaster reports error codition: ");
   }
   for (int i = 0; i < Feedback->Count; i++)
	  Log(" " + Feedback->Strings[i]);
   Log(" ");
   Snooze(10);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::ReportFailureOnDupOrBrokeChains(int tries, int cbrok, int cdupl)
{
   AnsiString errmsg = "I've encrypted this message " + IntToStr(tries) +
   " times and mixmaster failed everytime!   \nI encountered: \n";
   if (cbrok)
	  errmsg += IntToStr(cbrok) + " broken chain(s)   \n";
   if (cdupl)
	  errmsg += IntToStr(cdupl) + " duplicate chain(s)   \n";
   errmsg += "I suggest you update your remailer stats and try again!   \n";

   MsgBox(this, errmsg, MT_ERROR, MB_OK);
   Log(" Mixmaster is functioning improperly. ");
   Log(" Quitting! ");
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::MassageReferences(TStrings *message)
{
   TMsgManager *msgman = new TMsgManager(message);
   AnsiString refs = msgman->GetReferences(false);
   if (!refs.IsEmpty())
   {
	  int idx = msgman->DelHeader("References:");
	  assert(idx <= msgman->EOH());
	  assert(idx >= 0);
	  message->Insert(idx, "References: " + refs);
   }
   delete msgman;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Log(const AnsiString &text, bool force) // force after cancel
{
   Display->Lines->BeginUpdate();

   if (!Cancel() || force)
   {
	  Display->Lines->Add(text);
	  Display->Perform(EM_SCROLLCARET, 0, 0);
	  feedback->Add(text);
   }

   Display->Lines->EndUpdate();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::LogLast(const AnsiString &text, bool force) // force after cancel
{
   display->BeginUpdate();

   if (!Cancel() || force)
   {
	  display->Strings[display->Count - 1] = text;
   }

   display->EndUpdate();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::ShowStatus(AnsiString text)
{
#ifdef QS2
   UINT wm_msg = WM_SENDRETRSTATUS;
#else
   UINT wm_msg = WM_SHOWSTATUS;
#endif
   ((TForm*)Owner)->Perform(wm_msg, 0, (LPARAM)text.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::SaveSessionLog()
{
   TIniFile *ini = new TIniFile(AppIni());
   int maxlines = ini->ReadInteger("settings", "maxsmtplog", OptsMaxSMTPLogLines);
   delete ini;

   TStringList *lines = new TStringList();

   DB->LoadLog(SMTPLog, lines);
   lines->Add("");
   lines->AddStrings(Display->Lines);
   while (lines->Count && lines->Count > maxlines)
      lines->Delete(0);
   while (lines->Count && lines->Strings[0].Pos("Session:") != 1)
      lines->Delete(0);
   DB->SaveLog(SMTPLog, lines);

   delete lines;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::ExceptionHandler(TObject *Sender, Exception *E)
{
   //try
   //{
      SMTP->Abort();
      while (SMTP->Connected)
		 Application->ProcessMessages();
   //}
   //catch(...)
   //{
   //}

   Log(" Exception occurred: "+E->Message);
   Log(" Send aborted!");
   ShowExitInfo();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::WMLogAttachment(TMessage& Msg)
{
   AnsiString msg = (char*)Msg.LParam;
   Log(" attaching " + msg);
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::WMSendNextMessage(TMessage& Msg)
{
   if (showedexitinfo)
      return;
      
   SendNextMessage();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::CancelBtnClick(TObject *Sender)
{
   if (CancelBtn->Caption == "Close")
   {
	  ModalResult = mrOk;
	  return;
   }

   if (success) // too late to cancel
      return;

   MessageBeep(0);

   //try
   //{
      SMTP->Abort();
      while (SMTP->Connected)
         Application->ProcessMessages();
   //}
   //catch(...)
   //{
   //}

   canceled = true;
}
//---------------------------------------------------------------------------

bool __fastcall TSMTPDlg::Cancel()
{
   if (success) // too late to cancel
	  return false;

   if (!canceled)
	  return false;

   if (cancelhandled)
	  return true;

   cancelhandled = true;
   ProgressBar->Hide();

   //try
   //{
	  SMTP->Abort();
	  while (SMTP->Connected)
		 Application->ProcessMessages();
   //}
   //catch(...)
   //{
   //}

   Log("", true);
   Log("User canceled operation!", true);
   Log("", true);
   DB->UnQueueMessage(ProbQueue, messagekey, message);
   DB->QueueMessage(MailQueue, messagekey, message);
   messages->Insert(0, messagekey);
   ShowExitInfo();
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::PopupMenuPopup(TObject *Sender)
{
   Copy1->Enabled = Display->SelLength > 0;
   Selectall1->Enabled = Display->Text.Length() > 0;
   Resizedialog1->Enabled = BorderStyle == bsDialog;
   Locksize1->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Copy1Click(TObject *Sender)
{
   Display->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::VerboseClick(TObject *Sender)
{
   if (Verbose->Checked)
   {
      Display->Lines->Clear();
      Snooze(200);
      Display->Lines->BeginUpdate();
	  Display->Lines->AddStrings(verbose);
      Display->SelStart = 0; //Display->Text.Length();
      Display->Perform(EM_SCROLLCARET, 0, 0);
	  Display->Lines->EndUpdate();
   }
   else
   {
      Display->Lines->Clear();
	  Snooze(200);
      Display->Lines->BeginUpdate();
      Display->Lines->AddStrings(brief);
      Display->SelStart = 0; //Display->Text.Length();
      Display->Perform(EM_SCROLLCARET, 0, 0);
      Display->Lines->EndUpdate();

   }
}
//---------------------------------------------------------------------------

void __fastcall TSMTPDlg::Selectall1Click(TObject *Sender)
{
   Display->SelectAll();
}
//---------------------------------------------------------------------------

